# -*- coding: utf-8 -*-
import datetime
from functools import reduce
from django.db import models
from django.utils.functional import cached_property
from django.utils.translation import ugettext_lazy as _
from cms.models.fields import PageField
from cms.models.fields import PlaceholderField
from adminsortable.models import SortableMixin
from parler.models import TranslatedField
from djangocms_text_ckeditor.fields import HTMLField
from filer.fields.image import FilerImageField

from allink_core.core.models.fields import SortedM2MModelField
from allink_core.core.loading import get_class
from allink_core.core.models import (
    AllinkCategoryFieldsModel,
    AllinkBaseTranslatableModel,
    AllinkBaseAppContentPlugin,
    AllinkContactFieldsModel,
    AllinkAddressFieldsModel,
    AllinkBaseTranslatedFieldsModel,
)

AllinkLocationsManager = get_class('locations.managers', 'AllinkLocationsManager')


class BaseLocations(SortableMixin, AllinkContactFieldsModel, AllinkAddressFieldsModel,
                    AllinkCategoryFieldsModel, AllinkBaseTranslatableModel):
    TEASER_FIELD_FALLBACK_CONF = {
        'teaser_image': [
            {'model': 'self', 'field': 'teaser_image', },
            {'model': 'self', 'field': 'preview_image', },
        ],
        'teaser_title': [
            {'model': 'self', 'field': 'teaser_title', },
            {'model': 'self', 'field': 'title', },
        ],
        'teaser_technical_title': [
            {'model': 'self', 'field': 'teaser_technical_title', },
            {'model': 'self', 'field': 'opening_hours_display', },
        ],
        'teaser_description': [
            {'model': 'self', 'field': 'teaser_description', },
            {'model': 'self', 'field': 'lead', },
        ],
        'teaser_link_text': [
            {'model': 'self', 'field': 'teaser_link_text', },
            {'model': 'self', 'field': 'TEASER_LINK_TEXT', },
        ],
    }
    slug_source_field_name = 'title'

    title = TranslatedField(any_language=True)
    subtitle = TranslatedField()
    lead = TranslatedField()
    opening_hours_display = TranslatedField()
    preview_image = FilerImageField(
        verbose_name='Preview Image',
        blank=True,
        null=True,
        on_delete=models.PROTECT,
        related_name='%(app_label)s_%(class)s_preview_image',
    )
    lat = models.FloatField(
        'Latitude',
        blank=True,
        null=True
    )
    lng = models.FloatField(
        'Longitude',
        blank=True,
        null=True
    )
    map_link = models.URLField(
        'Map Link',
        help_text='This could be a <strong>Google Places</strong> or <strong>Directions</strong> link.',
        blank=True,
        null=True
    )
    commercial_register_entry = models.CharField(
        'Commercial register entry',
        blank=True,
        null=True,
        max_length=50,
    )
    mon = models.CharField(
        'Monday morning or whole day',
        help_text='Format: "(h)h:mm-(h)h:mm"', blank=True, max_length=100)
    tue = models.CharField(
        'Tuesday morning or whole day',
        help_text='Format: "(h)h:mm-(h)h:mm"', blank=True, max_length=100)
    wed = models.CharField(
        'Wednesday morning or whole day',
        help_text='Format: "(h)h:mm-(h)h:mm"', blank=True, max_length=100)
    thu = models.CharField(
        'Thursday morning or whole day',
        help_text='Format: "(h)h:mm-(h)h:mm"', blank=True, max_length=100)
    fri = models.CharField(
        'Friday morning or whole day',
        help_text='Format: "(h)h:mm-(h)h:mm"', blank=True, max_length=100)
    sat = models.CharField(
        'Saturday morning or whole day',
        help_text='Format: "(h)h:mm-(h)h:mm"', blank=True, max_length=100)
    sun = models.CharField(
        'Sunday morning or whole day',
        help_text='Format: "(h)h:mm-(h)h:mm"', blank=True, max_length=100)
    mon_afternoon = models.CharField(
        'Monday afternoon',
        help_text='Format: "(h)h:mm-(h)h:mm", only fill if location has a lunch break',
        blank=True, max_length=100
    )
    tue_afternoon = models.CharField(
        'Tuesday afternoon',
        help_text='Format: "(h)h:mm-(h)h:mm", only fill if location has a lunch break',
        blank=True, max_length=100
    )
    wed_afternoon = models.CharField(
        'Wednesday afternoon',
        help_text='Format: "(h)h:mm-(h)h:mm", only fill if location has a lunch break',
        blank=True, max_length=100
    )
    thu_afternoon = models.CharField(
        'Thursday afternoon',
        help_text='Format: "(h)h:mm-(h)h:mm", only fill if location has a lunch break',
        blank=True, max_length=100
    )
    fri_afternoon = models.CharField(
        'Friday afternoon',
        help_text='Format: "(h)h:mm-(h)h:mm", only fill if location has a lunch break',
        blank=True, max_length=100
    )
    sat_afternoon = models.CharField(
        'Saturday afternoon',
        help_text='Format: "(h)h:mm-(h)h:mm", only fill if location has a lunch break',
        blank=True, max_length=100
    )
    sun_afternoon = models.CharField(
        'Sunday afternoon',
        help_text='Format: "(h)h:mm-(h)h:mm", only fill if location has a lunch break',
        blank=True, max_length=100
    )
    sort_order = models.PositiveIntegerField(
        default=0,
        editable=False,
        db_index=True
    )
    header_placeholder = PlaceholderField(
        'locations_header',
        related_name='%(app_label)s_%(class)s_header_placeholder'
    )
    content_placeholder = PlaceholderField(
        'locations_content',
        related_name='%(app_label)s_%(class)s_content_placeholder'
    )

    objects = AllinkLocationsManager()

    class Meta:
        abstract = True
        app_label = 'locations'
        ordering = 'sort_order',
        verbose_name = 'Location'
        verbose_name_plural = 'Locations'

    def __str__(self):
        return '%s - %s' % (self.title, self.created.strftime('%d.%m.%Y'))

    def value_has_changed_for_fields(instance, fields):
        """
        Did any field values change since the last time they were saved?
        """
        if not instance.pk:  # if new
            return True
        old_values = instance.__class__._default_manager.filter(pk=instance.pk).values().get()
        for f in fields:
            if not getattr(instance, f) == old_values[f]:
                return True
        return False

    def geocode_location(self):
        """
        Update lat and lng fields based on address
        """
        from geopy import geocoders
        g = geocoders.GoogleV3()
        try:
            street_full = '{} {}'.format(self.street, self.street_nr) + self.street_additional
            place, (lat, lng) = g.geocode('%s, %s %s, %s' % (street_full, self.zip_code, self.place, self.country))
        except Exception as e:
            return "%s: %s" % (self, e)
        else:
            self.lat = lat
            self.lng = lng
        return True

    def is_currently_open(self):
        opening_times = [
            (self.mon, self.mon_afternoon),
            (self.tue, self.tue_afternoon),
            (self.wed, self.wed_afternoon),
            (self.thu, self.thu_afternoon),
            (self.fri, self.fri_afternoon),
            (self.sat, self.sat_afternoon),
            (self.sun, self.sun_afternoon)
        ]
        return self.opening_info(opening_times[datetime.date.today().weekday()])

    is_currently_open.boolean = True
    is_currently_open.short_description = 'Now open'

    def opening_info(self, times):
        """
        For the given times attr and the current time,
        is our store open?
        """
        morning = times[0]
        afternoon = times[1]

        try:
            current_time = datetime.datetime.today().time()
            morning_splited = morning.split('-')

            start_morning = morning_splited[0]
            end_morning = morning_splited[1]

            start_time = datetime.datetime.strptime(start_morning, '%H:%M').time()
            end_time = datetime.datetime.strptime(end_morning, '%H:%M').time()

            if start_time < current_time and end_time > current_time:
                return True

            elif afternoon:
                afternoon_splited = afternoon.split('-')

                start_afternoon = afternoon_splited[0]
                end_afternoon = afternoon_splited[1]

                start_time_afternoon = datetime.datetime.strptime(start_afternoon, '%H:%M').time()
                end_time_afternoon = datetime.datetime.strptime(end_afternoon, '%H:%M').time()

                if start_time_afternoon < current_time and end_time_afternoon > current_time:
                    return True
                else:
                    return False
            else:
                return False
        except (KeyError, IndexError):
            return False

    def has_opening_info(self):
        """
        Returns whether store has any filled in opening info
        If all fields are empty, returns False, else True
        """
        return reduce(
            lambda x, y: x or y,
            [
                self.mon,
                self.mon_afternoon,
                self.tue,
                self.tue_afternoon,
                self.wed,
                self.wed_afternoon,
                self.thu,
                self.thu_afternoon,
                self.fri,
                self.fri_afternoon,
                self.sat,
                self.sat_afternoon,
                self.sun,
                self.sun_afternoon
            ],
            False
        )

    @cached_property
    def opening_hours(self):
        """
        Return the opening hours for whole days in a summarized format

        Monday – Friday	09:00-19:00
        Saturday	09:00-17:00
        """

        opening_times = [
            (_('Monday'), self.mon, self.mon_afternoon),
            (_('Tuesday'), self.tue, self.tue_afternoon),
            (_('Wednesday'), self.wed, self.wed_afternoon),
            (_('Thursday'), self.thu, self.thu_afternoon),
            (_('Friday'), self.fri, self.fri_afternoon),
            (_('Saturday'), self.sat, self.sat_afternoon),
            (_('Sunday'), self.sun, self.sun_afternoon)
        ]

        opening_hours = []
        for i, opening_time in enumerate(opening_times):
            day, morning, afternoon = opening_time
            morning = morning.replace('-', ' – ')
            afternoon = afternoon.replace('-', ' – ')

            if not morning and not afternoon:
                day = {}
                opening_hours.append(day)
            elif len(opening_hours) \
                      and morning == list(map(lambda x: x.get('morning'), opening_hours))[-1] \
                    and afternoon == list(map(lambda x: x.get('afternoon'), opening_hours))[-1]:
                opening_hours[-1]['end_day'] = day
            else:
                day = {
                    'start_day': day,
                    'end_day': day,
                    'morning': morning,
                    'afternoon': afternoon,
                }
                opening_hours.append(day)

        return opening_hours

    @cached_property
    def gmaps_link(self):
        """
        Returns google maps link with query of current store
        """
        return ("https://www.google.ch/maps?q=%(name)s+%(street)s+%(zip_code)s+%(place)s" % {
            'name': self.title,
            'street': '{} {} {}'.format(self.street, self.street_nr, self.street_additional),
            'zip_code': self.zip_code,
            'place': self.place
        }).replace(' ', '+')


class BaseLocationsTranslation(AllinkBaseTranslatedFieldsModel):
    master = models.ForeignKey(
        'locations.Locations',
        on_delete=models.CASCADE,
        related_name='translations',
        null=True
    )
    title = models.CharField(
        max_length=255
    )
    subtitle = models.CharField(
        'Subtitle',
        max_length=255,
        blank=True,
        null=True,
    )
    lead = HTMLField(
        'Lead Text',
        help_text='Teaser text that in some cases is used in the list view and/or in the detail view.',
        blank=True,
        null=True,
    )
    opening_hours_display = HTMLField(
        'Opening hours',
        help_text=('This Text will be used to show the Opening hours on the location detail page. '
                   'If provided, the detailed opening hours will be overriden.'),
        blank=True,
        null=True,
    )

    class Meta:
        abstract = True
        app_label = 'locations'


class BaseLocationsAppContentPlugin(AllinkBaseAppContentPlugin):
    ZOOM_LEVEL_CHOICES = (
        (0, 0),
        (1, 1),
        (2, 2),
        (3, 3),
        (4, 4),
        (5, 5),
        (6, 6),
        (7, 7),
        (8, 8),
        (9, 9),
        (10, 10),
        (11, 11),
        (12, 12),
        (13, 13),
        (14, 14),
        (15, 15),
        (16, 16),
        (17, 17),
        (18, 18),
    )

    manual_entries = SortedM2MModelField(
        'locations.Locations',
        blank=True,
        help_text='Select and arrange specific entries, or, leave blank to select all. (If '
                  'manual entries are selected the category filtering will be applied as well.)'
    )
    apphook_page = PageField(
        verbose_name='Apphook Page',
        null=True,
        blank=True,
        on_delete=models.PROTECT,
        help_text='If provided, this Apphook-Page will be used to generate the detail link.',
    )
    zoom_level = models.IntegerField(
        'Zoom Level',
        help_text='The higher the number, the more we zoom in.',
        choices=ZOOM_LEVEL_CHOICES,
        default=14
    )

    def save(self, *args, **kwargs):
        super(BaseLocationsAppContentPlugin, self).save(*args, **kwargs)

    class Meta:
        abstract = True
        app_label = 'locations'



# Generated by Django 2.1.8 on 2019-07-02 17:18

from django.conf import settings
import django.contrib.postgres.fields
from django.db import migrations, models
import django.db.models.deletion
import filer.fields.file
import filer.fields.image


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.FILER_IMAGE_MODEL),
        ('cms', '0022_auto_20180620_1551'),
        ('filer', '0011_auto_20190418_0137'),
    ]

    operations = [
        migrations.CreateModel(
            name='AllinkContentColumnPlugin',
            fields=[
                ('cmsplugin_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, related_name='allink_content_allinkcontentcolumnplugin', serialize=False, to='cms.CMSPlugin')),
                ('title', models.CharField(blank=True, max_length=255, null=True, verbose_name='Title')),
                ('order_mobile', models.IntegerField(blank=True, help_text='Some columns should be ordered differently on mobile devices when columns are stacked vertically. This option allows you to define the position of the this column.<br><br>Note: Columns ordering is ascending (lowest number displayed first)', null=True, verbose_name='Order Mobile')),
                ('alignment_horizontal_mobile', models.CharField(blank=True, choices=[('left', 'Left'), ('center', 'Center'), ('right', 'Right')], help_text='This option overrides the projects default alignment for mobile. (Usually "left")', max_length=50, null=True, verbose_name='Alignment horizontal mobile')),
                ('alignment_horizontal_desktop', models.CharField(blank=True, choices=[('left', 'Left'), ('center', 'Center'), ('right', 'Right')], help_text='This option overrides the projects default alignment for desktop. (Usually "left")', max_length=50, null=True, verbose_name='Alignment horizontal desktop')),
                ('alignment_vertical_desktop', models.CharField(blank=True, choices=[('top', 'Top'), ('middle', 'Middle'), ('bottom', 'Bottom')], help_text='This option overrides the projects default alignment for desktop. (Usually "top")', max_length=50, null=True, verbose_name='Alignment vertical desktop')),
            ],
            options={
                'abstract': False,
            },
            bases=('cms.cmsplugin',),
        ),
        migrations.CreateModel(
            name='AllinkContentPlugin',
            fields=[
                ('title', models.CharField(blank=True, help_text='The section title', max_length=255, null=True, verbose_name='Title')),
                ('title_size', models.CharField(max_length=50, verbose_name='Section Title Size')),
                ('template', models.CharField(help_text='Choose a template.', max_length=50, verbose_name='Template')),
                ('container_enabled', models.BooleanField(default=True, help_text='If checked, an inner container with a maximum width is added', verbose_name='Activate "container"')),
                ('inverted_colors_enabled', models.BooleanField(default=False, help_text='If checked, the predefined inverted text colors are applied (suitable when using a background image/video)', verbose_name='Activate "inverted text colors"')),
                ('overlay_enabled', models.BooleanField(default=False, help_text='If checked, a predefined overlay background gradient/color is applied.<br><strong>Important:</strong> Only applied when used in combination with a background image/video/color', verbose_name='Activate "overlay"')),
                ('bg_color', models.CharField(blank=True, max_length=50, null=True, verbose_name='Set a predefined background color')),
                ('dynamic_height_enabled', models.BooleanField(default=False, help_text='If checked, the section\'s height will grow depending on the height of its children.<br>Note: This option is being ignored when "full height" or "parallax" are enabled.', verbose_name='Activate dynamic height')),
                ('anchor', models.CharField(blank=True, help_text='ID of this content section which can be used for anchor reference from links.<br>Note: Only letters, numbers and hyphen. No spaces or special chars.', max_length=255, verbose_name='ID')),
                ('ignore_in_pdf', models.BooleanField(default=False, help_text='If checked, this content section will be ignored when generting a pdf.', verbose_name='Ignore for pdf export')),
                ('project_css_classes', django.contrib.postgres.fields.ArrayField(base_field=models.CharField(blank=True, max_length=50, null=True), blank=True, null=True, size=None)),
                ('project_on_screen_effect', models.CharField(blank=True, default='default', max_length=50, null=True)),
                ('cmsplugin_ptr', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, related_name='allink_content_allinkcontentplugin', serialize=False, to='cms.CMSPlugin')),
                ('project_css_spacings_top_bottom', models.CharField(blank=True, help_text='Choose a spacing (top and bottom).', max_length=50, null=True, verbose_name='Spacings')),
                ('project_css_spacings_top', models.CharField(blank=True, help_text='Choose a top spacing.', max_length=50, null=True, verbose_name='Spacings top')),
                ('project_css_spacings_bottom', models.CharField(blank=True, help_text='Choose a bottom spacing.', max_length=50, null=True, verbose_name='Spacings bottom')),
                ('bg_image_outer_container', filer.fields.image.FilerImageField(blank=True, help_text='Optional: Set a background image for the content section.', null=True, on_delete=django.db.models.deletion.PROTECT, related_name='allink_content_allinkcontentplugin_bg_image', to=settings.FILER_IMAGE_MODEL, verbose_name='Background-Image')),
                ('video_file', filer.fields.file.FilerFileField(blank=True, help_text='Recommended video settings:<br><br>Format: mp4<br>Codec: H.264<br>Target Bitrate: 2 (video loads quick and runs smooth)<br>Audio: Not recommended (no audio = smaller file size and less annoyed visitors)<br>File size: Dependent of video length. Generally speaking: Less is more.', null=True, on_delete=django.db.models.deletion.PROTECT, related_name='content_video_file', to='filer.File', verbose_name='Source')),
                ('video_mobile_image', filer.fields.image.FilerImageField(blank=True, help_text='The image that is being displayed on mobile devices instead of the video.', null=True, on_delete=django.db.models.deletion.PROTECT, related_name='content_video_mobile_image', to=settings.FILER_IMAGE_MODEL, verbose_name='Mobile Image')),
                ('video_poster_image', filer.fields.image.FilerImageField(blank=True, help_text='This image is displayed while the video is loading. Ideally, use an <strong>exact screen capture image</strong> of the very first frame of the video for best results.', null=True, on_delete=django.db.models.deletion.PROTECT, related_name='content_video_poster_image', to=settings.FILER_IMAGE_MODEL, verbose_name='Video Start Image')),
            ],
            options={
                'abstract': False,
            },
            bases=('cms.cmsplugin',),
        ),
    ]



# -*- coding: utf-8 -*-
"""
use core:
from allink_core.apps.dummy_app.managers import *  # noqa
"""
from allink_core.apps.dummy_app.managers import AllinkDummyAppQuerySet


class AllinkDummyAppQuerySet(AllinkDummyAppQuerySet):
    def new(self):
        return self.filter(status=1)


AllinkDummyAppManager = AllinkDummyAppQuerySet.as_manager



# Generated by Django 2.1.10 on 2019-10-02 19:51

from django.db import migrations
import django.db.models.deletion
import parler.fields


class Migration(migrations.Migration):

    dependencies = [
        ('allink_categories', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='allinkcategorytranslation',
            name='master',
            field=parler.fields.TranslationsForeignKey(editable=False, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='translations', to='allink_categories.AllinkCategory'),
        ),
    ]



# -*- coding: utf-8 -*-
from django.db import models
from aldryn_translation_tools.models import TranslationHelperMixin
from parler.models import TranslatableModel, TranslatedFieldsModel
from parler.models import TranslatedField
from model_utils.models import TimeStampedModel

from allink_core.core.models.managers import AllinkBaseModelManager
from allink_core.core.models.fields_model import (
    AllinkStatusFieldsModel,
    AllinkSEOFieldsModel,
    AllinkSEOTranslatedFieldsModel,
    AllinkTeaserFieldsModel,
    AllinkTeaserTranslatedFieldsModel,
)
from allink_core.core.models.mixins import (
    AllinkTranslatedAutoSlugifyMixin,
    AllinkInvalidatePlaceholderCacheMixin,
    AllinkMetaTagMixin,
    AllinkTeaserMixin,
    AllinkDetailMixin
)

__all__ = [
    'AllinkBaseTranslatableModel',
    'AllinkBaseTranslatedFieldsModel',
]


class AllinkBaseTranslatableModel(TranslationHelperMixin, AllinkTranslatedAutoSlugifyMixin,
                                  AllinkInvalidatePlaceholderCacheMixin, AllinkMetaTagMixin, AllinkTeaserMixin,
                                  AllinkStatusFieldsModel, AllinkDetailMixin, TimeStampedModel, AllinkSEOFieldsModel,
                                  AllinkTeaserFieldsModel, TranslatableModel):
    """
    Base model class for apps with detail view

    - every concrete implementation expects the following attribute:
        slug_source_field_name = 'title'  # does not have to be 'title'

    - get_absolute_url expects a slug field defined

    """

    objects = AllinkBaseModelManager()
    slug = TranslatedField(any_language=True)

    class Meta:
        abstract = True


class AllinkBaseTranslatedFieldsModel(AllinkSEOTranslatedFieldsModel, AllinkTeaserTranslatedFieldsModel,
                                      TranslatedFieldsModel):
    """
    Base model class for apps with detail view
    """

    slug = models.SlugField(
        'Slug',
        max_length=255,
        default='',
        blank=True,
        help_text='Leave blank to auto-generate a unique slug.'
    )

    class Meta:
        abstract = True



# -*- coding: utf-8 -*-
import datetime
from django.test.testcases import TestCase
from ..models import News
from .factories import NewsFactory
from freezegun import freeze_time

today = datetime.date(2019, 3, 23)

@freeze_time(today)
class NewsManagerTestCase(TestCase):

    def setUp(self):
        # published
        self.active = NewsFactory.create_batch(3, title='active')
        self.active_and_published = NewsFactory.create_batch(3, title='active_and_published', start=datetime.date(2019, 3, 22), end=None)
        self.active_and_published_with_end = NewsFactory.create_batch(3, title='active_and_published_with_end', end=datetime.date(2019, 3, 23))
        self.active_and_published_start_is_today = NewsFactory.create_batch(3, title='active_and_published_start_is_today', start=datetime.date(2019, 3, 23), end=None)
        self.active_and_published_end_is_today = NewsFactory.create_batch(3, title='active_and_published_end_is_today', start=datetime.date(2019, 3, 22), end=datetime.date(2019, 3, 23))

        # not published
        self.inactive = NewsFactory.create_batch(2, title='inactive', status=News.INACTIVE)
        self.active_but_not_yet_published = NewsFactory.create_batch(2, title='active_but_not_yet_published', start=datetime.date(2019, 3, 24), end=datetime.date(2019, 3, 27))
        self.active_but_not_yet_published_no_end = NewsFactory.create_batch(2, title='active_but_not_yet_published_no_end', start=datetime.date(2019, 3, 24), end=None)
        self.active_but_outdated_no_start= NewsFactory.create_batch(2, title='active_but_outdated_no_start', start=datetime.date(2019, 3, 24), end=None)
        self.active_and_outdated = NewsFactory.create_batch(2, title='active_and_outdated', start=datetime.date(2019, 3, 21), end=datetime.date(2019, 3, 22))


    def test_active(self):
        all_active = self.active + self.active_and_published_with_end + self.active_and_published + self.active_and_published_start_is_today + self.active_and_published_end_is_today
        self.assertCountEqual(all_active, list(News.objects.active()))

    def test_published_fields_empty(self):
        expected_result =  self.active + self.inactive
        self.assertCountEqual(expected_result, list(News.objects.published_fields_empty()))

    def test_published_with_startdate_either_and_no_enddate(self):
        expected_result = self.active_and_published + self.active_and_published_start_is_today
        self.assertCountEqual(expected_result, list(News.objects.published_with_startdate(start=today)))

    def test_published_with_enddate_and_no_startdate(self):
        expected_result = self.active_and_published_with_end
        self.assertCountEqual(expected_result, list(News.objects.published_with_enddate(end=today)))



# Generated by Django 2.2.7 on 2020-02-05 14:27

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('allink_teaser', '0004_allinkteaserplugin_softpage_enabled'),
    ]

    operations = [
        migrations.AlterField(
            model_name='allinkteaserplugin',
            name='softpage_enabled',
            field=models.BooleanField(default=False, help_text='If checked, the detail view of an entry will be displayed in a "softpage". Otherwise the page will be reloaded.', verbose_name='Show detailed information in Softpage'),
        ),
    ]



# -*- coding: utf-8 -*-
from itertools import chain

from django import forms
from django.template.loader import render_to_string
from django.urls import NoReverseMatch, reverse
from django.utils.encoding import force_text
from django.utils.html import conditional_escape
from django.contrib.admin.widgets import AdminFileWidget
from django.utils.safestring import mark_safe
from django.conf import settings
import sortedm2m.forms

from allink_core.core.utils import get_project_color_choices
from allink_core.core.admin.mixins import AllinkMediaAdminMixin


class AdminPdfThumnailWidget(AdminFileWidget):
    def render(self, name, value, attrs=None):
        output = []

        if value:
            output.append('<img alt="%s" src="%s" height="200"/>' % (value.url, value.url,))
        else:
            output.append('Thumbnail is created when publication is saved. And a PDF document is assigned.')

        # This is commented out b/c maybe you want to be able to override the thumbnail?
        # output.append(super(AdminFileWidget, self).render(name, value, attrs))

        return mark_safe(''.join(output))


class Icon(forms.widgets.TextInput):
    def render(self, name, value, attrs=None, **kwargs):
        input_html = super(Icon, self).render(name, value, attrs=attrs, **kwargs)
        if value is None:
            value = ''
        iconset = value.split('-')[0] if value and '-' in value else ''
        iconset_prefexes = [s[1] for s in settings.PROJECT_ICONSETS]
        if len(settings.PROJECT_ICONSETS) and iconset not in iconset_prefexes:
            # invalid iconset! maybe because the iconset was removed from
            # the project. set it to the first in the list.
            iconset = settings.PROJECT_ICONSETS[0][1]
        from django.template.loader import render_to_string
        rendered = render_to_string(
            'admin/allink_button_link/widgets/icon.html',
            {
                'input_html': input_html,
                'value': value,
                'name': name,
                'iconset': iconset,
                'is_required': self.is_required,
                'iconsets': settings.PROJECT_ICONSETS,
            },
        )
        return rendered


class SpectrumColorPicker(AllinkMediaAdminMixin, forms.widgets.TextInput):
    """
    Based on Brian Grinstead's Spectrum - http://bgrins.github.com/spectrum/
    This widget is used to select a Project Color. With some few options
    in the pushed colorFields, it could be used more flexible if needed.
    """

    def __init__(self, *args, **kwargs):
        self.default = kwargs.pop('default', None)
        super(SpectrumColorPicker, self).__init__(*args, **kwargs)

    def _get_project_color_choices(self):
        palette = ",".join("'%s'" % color for color in get_project_color_choices())
        if self.default:
            palette = "'%s'," % self.default + palette
        else:
            palette = palette + ",'transparent'"
        return palette

    def _render_js(self, _id, value):
        js = """
            <script type="text/javascript">
                document.addEventListener("DOMContentLoaded", function(event) {
                    if (!window.colorFields) {
                        window.colorFields = [];
                    }

                    window.colorFields.push({
                        id: '#%s',
                        color: "%s",
                        showPaletteOnly: true,
                        palette:[%s],
                        localStorageKey: "projectcolors"
                    });
                });
            </script>""" % (_id, value, self._get_project_color_choices())
        return js

    def render(self, name, value, attrs=None, renderer=None):
        if 'id' not in attrs:
            attrs['id'] = "id_%s" % name
        if value:
            for key, val in get_project_color_choices().items():
                if val == value:
                    value = key
                    break
        rendered = super(SpectrumColorPicker, self).render(name, value, attrs=attrs, renderer=None)
        return mark_safe(rendered + self._render_js(attrs['id'], value))


class SearchSelectWidget(AllinkMediaAdminMixin, forms.widgets.Select):

    def __init__(self, attrs=None, choices=()):
        class_attrs = {"data-live-search": "true", 'class': 'selectpicker'}
        if attrs:
            attrs.update(class_attrs)
        else:
            attrs = class_attrs
        super(SearchSelectWidget, self).__init__(attrs)


class SortedM2MWidget(sortedm2m.forms.SortedCheckboxSelectMultiple):
    """
    Copied from https://github.com/divio/aldryn-common

    Copied because this package is no longer maintained.
    https://github.com/divio/aldryn-common/commit/6582754be67390d056d4aaa1f799c183e808c914

    """

    template = 'widgets/sortedm2m_widget.html'

    def render(self, name, value, attrs=None, choices=()):
        # TODO: make a pull request to sortedm2m to make it easy to override the template
        # TODO: make a pull request to sortedm2m to make it easy to add the addtional
        # TODO: link to admin (or integrate this whole widget)
        if value is None:
            value = []
        has_id = attrs and 'id' in attrs
        final_attrs = self.build_attrs(attrs, name=name)

        # Normalize to strings
        str_values = [force_text(v) for v in value]

        selected = []
        unselected = []

        # get the admin link
        rel_to = self.choices.queryset.model  # this is hacky
        info = (rel_to._meta.app_label, rel_to._meta.object_name.lower())

        for i, (option_value, option_label) in enumerate(chain(self.choices, choices)):
            # If an ID attribute was given, add a numeric index as a suffix,
            # so that the checkboxes don't all have the same ID attribute.
            if has_id:
                final_attrs = dict(final_attrs, id='%s_%s' % (attrs['id'], i))
                label_for = ' for="%s"' % conditional_escape(final_attrs['id'])
            else:
                label_for = ''

            cb = forms.CheckboxInput(final_attrs, check_test=lambda value: value in str_values)
            option_value = force_text(option_value)
            rendered_cb = cb.render(name, option_value)
            option_label = conditional_escape(force_text(option_label))

            try:
                admin_url = reverse(
                    'admin:%s_%s_change' % info,
                    # current_app=self.admin_site.name,
                    args=(option_value,),
                )
            except NoReverseMatch:
                info = (
                    self.admin_site.root_path,
                    rel_to._meta.app_label,
                    rel_to._meta.object_name.lower(),
                    option_value,
                )
                admin_url = '%s%s/%s/change/%s' % info
            # admin_url = (
            #     u'<a href="%s" style="margin-left:10px;"><img src="%sadmin/img/admin/icon_changelink.gif" '
            #     u'width="10" height="10" alt="View current"/></a>'
            # ) % (url, settings.STATIC_URL)
            item = {
                'label_for': label_for,
                'rendered_cb': rendered_cb,
                'option_label': option_label,
                'option_value': option_value,
                'admin_url': admin_url,
            }

            if option_value in str_values:
                selected.append(item)
            else:
                unselected.append(item)

        # re-order `selected` array according str_values which is a set of `option_value`s in the order they should
        # be shown on screen
        ordered = []
        for value in str_values:
            for select in selected:
                if value == select['option_value']:
                    ordered.append(select)
        selected = ordered

        html = render_to_string(
            self.template,
            {'selected': selected, 'unselected': unselected})
        return mark_safe(html)



from django.utils.translation import ugettext_lazy as _
from cms.plugin_base import CMSPluginBase
from cms.plugin_pool import plugin_pool

from .models import AllinkQuotePlugin


@plugin_pool.register_plugin
class CMSAllinkQuotePlugin(CMSPluginBase):
    name = _('Quote')
    module = _('allink modules')
    render_template = 'allink_quote/content.html'
    model = AllinkQuotePlugin

    def render(self, context, instance, placeholder):
        context['instance'] = instance
        context['placeholder'] = placeholder
        return context



import sys
import traceback
from importlib import import_module

from django.apps import apps
from django.apps.config import MODELS_MODULE_NAME
from django.conf import settings
from django.core.exceptions import AppRegistryNotReady

from allink_core.core.exceptions import (AppNotFoundError, ClassNotFoundError, ModuleNotFoundError)


def get_class(module_label, classname, module_prefix='allink_core.apps'):
    """
    Dynamically import a single class from the given module.
    This is a simple wrapper around `get_classes` for the case of loading a
    single class.
    Args:
        module_label (str): Module label comprising the app label and the
            module name, separated by a dot.  For example, 'news.forms'.
        classname (str): Name of the class to be imported.
    Returns:
        The requested class object or `None` if it can't be found
    """
    return get_classes(module_label, [classname], module_prefix)[0]


def get_classes(module_label, classnames, module_prefix='allink_core.apps'):
    """
    Dynamically import a list of classes from the given module.
    This works by looping over ``INSTALLED_APPS`` and looking for a match
    against the passed module label.  If the requested class can't be found in
    the matching module, then we attempt to import it from the corresponding
    core app.
    This is very similar to ``django.db.models.get_model`` function for
    dynamically loading models.  This function is more general though as it can
    load any class from the matching app, not just a model.
    Args:
        module_label (str): Module label comprising the app label and the
            module name, separated by a dot.  For example, 'news.forms'.
        classname (str): Name of the class to be imported.
    Returns:
        The requested class object or ``None`` if it can't be found
    Examples:
        Load a single class:
        >>> get_class('apps.news.forms', 'ProductForm')
        allink.apps.apps.news.forms.ProductForm
        Load a list of classes:
        >>> get_classes('apps.news.forms',
        ...             ['ProductForm', 'StockRecordForm'])
        [allink.apps.apps.news.forms.ProductForm,
         allink.apps.apps.news.forms.StockRecordForm]
    Raises:
        AppNotFoundError: If no app is found in ``INSTALLED_APPS`` that matches
            the passed module label.
        ImportError: If the attempted import of a class raises an
            ``ImportError``, it is re-raised
    """
    if '.' not in module_label:
        # Importing from top-level modules is not supported, e.g.
        # get_class('shipping', 'Scale'). That should be easy to fix,
        # but @maikhoepfel had a stab and could not get it working reliably.
        # Overridable classes in a __init__.py might not be a good idea anyway.
        raise ValueError(
            "Importing from top-level modules is not supported")

    # import from allink package (should succeed in most cases)
    # e.g. 'allink.apps.apps.news.forms'
    allink_module_label = "%s.%s" % (module_prefix, module_label)
    allink_module = _import_module(allink_module_label, classnames)

    # returns e.g. 'allink.apps.apps.news',
    # 'yourproject.apps.apps.news' or 'apps.news',
    # depending on what is set in INSTALLED_APPS
    installed_apps_entry, app_name = _find_installed_apps_entry(module_label)
    if installed_apps_entry.startswith('%s.' % module_prefix):
        # The entry is obviously an allink one, we don't import again
        local_module = None
    else:
        # Attempt to import the classes from the local module
        # e.g. 'yourproject.apps.news.forms'
        sub_module = module_label.replace(app_name, '', 1)
        local_module_label = installed_apps_entry + sub_module
        local_module = _import_module(local_module_label, classnames)

    allink_move_module = None

    if allink_module is allink_move_module is local_module is None:
        # This intentionally doesn't raise an ImportError, because ImportError
        # can get masked in complex circular import scenarios.
        raise ModuleNotFoundError(
            "The module with label '%s' could not be imported. This either"
            "means that it indeed does not exist, or you might have a problem"
            " with a circular import." % module_label
        )

    # return imported classes, giving preference to ones from the local package
    return _pluck_classes([local_module, allink_module, allink_move_module], classnames)


def _import_module(module_label, classnames):
    """
    Imports the module with the given name.
    Returns None if the module doesn't exist, but propagates any import errors.
    """
    try:
        return __import__(module_label, fromlist=classnames)
    except ImportError:
        # There are 2 reasons why there could be an ImportError:
        #
        #  1. Module does not exist. In that case, we ignore the import and
        #     return None
        #  2. Module exists but another ImportError occurred when trying to
        #     import the module. In that case, it is important to propagate the
        #     error.
        #
        # ImportError does not provide easy way to distinguish those two cases.
        # Fortunately, the traceback of the ImportError starts at __import__
        # statement. If the traceback has more than one frame, it means that
        # application was found and ImportError originates within the local app
        __, __, exc_traceback = sys.exc_info()
        frames = traceback.extract_tb(exc_traceback)
        if len(frames) > 1:
            raise


def _pluck_classes(modules, classnames):
    """
    Gets a list of class names and a list of modules to pick from.
    For each class name, will return the class from the first module that has a
    matching class.
    """
    klasses = []
    for classname in classnames:
        klass = None
        for module in modules:
            if hasattr(module, classname):
                klass = getattr(module, classname)
                break
        if not klass:
            packages = [m.__name__ for m in modules if m is not None]
            raise ClassNotFoundError("No class '%s' found in %s" % (
                classname, ", ".join(packages)))
        klasses.append(klass)
    return klasses


def _get_installed_apps_entry(app_name):
    """
    Given an app name (e.g. 'news'), walk through INSTALLED_APPS
    and return the first match, or None.
    This does depend on the order of INSTALLED_APPS and will break if
    e.g. 'apps.news' comes before 'news' in INSTALLED_APPS.
    """
    for installed_app in settings.INSTALLED_APPS:
        # match root-level apps ('news') or apps with same name at end
        # ('shop.news'), but don't match 'fancy_news'
        if installed_app == app_name or installed_app.endswith('.' + app_name):
            return installed_app
    return None


def _find_installed_apps_entry(module_label):
    """
    Given a module label, finds the best matching INSTALLED_APPS entry.
    This is made trickier by the fact that we don't know what part of the
    module_label is part of the INSTALLED_APPS entry. So we try all possible
    combinations, trying the longer versions first. E.g. for
    'apps.news.forms', 'apps.news' is attempted before
    'dashboard'
    """
    modules = module_label.split('.')
    # if module_label is 'apps.news.forms.widgets', combinations
    # will be ['apps.news.forms', 'apps.news', 'dashboard']
    combinations = [
        '.'.join(modules[:-count]) for count in range(1, len(modules))]
    for app_name in combinations:
        entry = _get_installed_apps_entry(app_name)
        if entry:
            return entry, app_name
    raise AppNotFoundError(
        "Couldn't find an app to import %s from" % module_label)


def get_model(app_label, model_name):
    """
    Fetches a Django model using the app registry.
    This doesn't require that an app with the given app label exists,
    which makes it safe to call when the registry is being populated.
    All other methods to access models might raise an exception about the
    registry not being ready yet.
    Raises LookupError if model isn't found.
    """
    try:
        return apps.get_model(app_label, model_name)
    except AppRegistryNotReady:
        if apps.apps_ready and not apps.models_ready:
            # If this function is called while `apps.populate()` is
            # loading models, ensure that the module that defines the
            # target model has been imported and try looking the model up
            # in the app registry. This effectively emulates
            # `from path.to.app.models import Model` where we use
            # `Model = get_model('app', 'Model')` instead.
            app_config = apps.get_app_config(app_label)
            # `app_config.import_models()` cannot be used here because it
            # would interfere with `apps.populate()`.
            import_module('%s.%s' % (app_config.name, MODELS_MODULE_NAME))
            # In order to account for case-insensitivity of model_name,
            # look up the model through a private API of the app registry.
            return apps.get_registered_model(app_label, model_name)
        else:
            # This must be a different case (e.g. the model really doesn't
            # exist). We just re-raise the exception.
            raise


def is_model_registered(app_label, model_name):
    """
    Checks whether a given model is registered. This is used to only
    register allink models if they aren't overridden by a forked app.
    """
    try:
        apps.get_registered_model(app_label, model_name)
    except LookupError:
        return False
    else:
        return True


def unregister_cms_menu(menu_cls):
    """
    Django CMS doesn't offer a function to unregister a cms menu
    Allows overriding cms menu in allink_apps
    """
    from menus.base import Menu
    from menus.menu_pool import menu_pool
    assert issubclass(menu_cls, Menu)
    if menu_cls.__name__ in menu_pool.menus:
        # Note: menu_cls should still be the menu CLASS at this point.
        del menu_pool.menus[menu_cls.__name__]


def unregister_cms_apps(app):
    """
    Django CMS doesn't offer a function to unregister a cms app
    Allows overriding cms apps in allink_apps
    """
    from cms.apphook_pool import apphook_pool

    if app.__name__ in apphook_pool.apps:
        del apphook_pool.apps[app.__name__]



# -*- coding: utf-8 -*-
from django.utils.translation import ugettext_lazy as _
from allink_core.core.utils import get_additional_choices

####################################################################################

#  BLANK

BLANK_CHOICE = (('', '---------'),)


####################################################################################

#  SALUTATION

MR = 1
MRS = 2

SALUTATION_CHOICES = (
    (MR, _('Mr.')),
    (MRS, _('Mrs.')),
)

####################################################################################

# POSITIONING

LEFT = 'left'
CENTER = 'center'
RIGHT = 'right'

HORIZONTAL_ALIGNMENT_CHOICES = (
    (LEFT, 'Left'),
    (CENTER, 'Center'),
    (RIGHT, 'Right'),
)

TOP = 'top'
MIDDLE = 'middle'
BOTTOM = 'bottom'

VERTICAL_ALIGNMENT_CHOICES = (
    (TOP, 'Top'),
    (MIDDLE, 'Middle'),
    (BOTTOM, 'Bottom'),
)


####################################################################################

# TYPOGRAPHY

# Used for section headings for our 'Content Plugin' and 'App Content Plugin'

# H1 = 'h1'
# H2 = 'h2'
# H3 = 'h3'
#
# TITLE_CHOICES = (
#     (H1, _('Title Large')),
#     (H2, _('Title Medium')),
#     (H3, _('Title Small')),
# )


####################################################################################

# VIDEO SERVICES
YOUTUBE = 'youtube'
VIMEO = 'vimeo'

VIDEO_SERVICE_CHOICES = (
    (YOUTUBE, 'Youtube'),
    (VIMEO, 'Vimeo'),
)


####################################################################################

# BUTTON / TEXT

# Used in the 'Button / Link' plugin

SIZE_CHOICES = (
    ('sm', 'Small'),
    ('md', 'Medium'),
    ('lg', 'Large'),
)

CONTEXT_CHOICES = get_additional_choices('BUTTON_CONTEXT_CHOICES')

CONTEXT_DEFAULT = 'default'

BUTTON_CONTEXT_CHOICES = (
    (CONTEXT_DEFAULT, 'Default',),
) + CONTEXT_CHOICES

BUTTON_CONTEXT_DEFAULT = 'default'

TEXT_LINK_CONTEXT_CHOICES = (
    ('', 'Default',),
) + CONTEXT_CHOICES + (
    ('muted ', 'Muted',),
)

TEXT_LINK_CONTEXT_DEFAULT = ''

NEW_WINDOW = 1
SOFTPAGE = 2
FORM_MODAL = 4
IMAGE_MODAL = 5
DEFAULT_MODAL = 6

TARGET_CHOICES = (
    (NEW_WINDOW, 'New window'),
    (SOFTPAGE, 'Softpage'),
    (FORM_MODAL, 'Lightbox (Forms)'),
    (IMAGE_MODAL, 'Lightbox (Image)'),
    (DEFAULT_MODAL, 'Lightbox (Default)'),
)

####################################################################################

# IMAGE

RATIO_CHOICES = (
    ('3-2', '3:2'),
    ('2-1', '2:1'),
    ('4-3', '4:3'),
    ('1-1', '1:1'),
    ('16-9', '16:9'),
)
RATIO_CHOICES_ORIG = RATIO_CHOICES + (('x-y', 'Original'),)

IMAGE_WIDTH_ALIAS_CHOICES = ()



import io
import factory
from django.core.files import File
from django.contrib.auth import get_user_model
from PIL import Image as PILimage

from filer.models import Image

__all__ = ['UserFactory', 'FilerImageFactory']


class UserFactory(factory.DjangoModelFactory):
    username = factory.Sequence(lambda n: 'allink user nummer %d' % n)
    email = factory.Sequence(lambda n: 'example_%s@example.com' % n)
    first_name = factory.Sequence(lambda n: 'Fritzli %d' % n)
    last_name = factory.Sequence(lambda n: 'Bühler %d' % n)
    password = factory.PostGenerationMethodCall('set_password', 'itcrowd')
    is_active = True
    is_superuser = True
    is_staff = True

    class Meta:
        model = get_user_model()


class FilerImageFactory(factory.DjangoModelFactory):
    """
    Create a Filer Image for filer.fields.image.FilerImageField.
    """

    class Meta:
        model = Image

    owner = factory.SubFactory(UserFactory)
    original_filename = factory.Faker("file_name", category="image")

    @factory.lazy_attribute
    def file(self):
        """
        Fill file field with generated image on the fly by PIL.
        Generated image is just a blank image of 100x100 with plain blue color.
        Returns:
            django.core.files.File: File object.
        """
        # ImageField (both django's and factory_boy's) require PIL.
        thumb = PILimage.new("RGB", (100, 100), "blue")
        thumb_io = io.BytesIO()
        thumb.save(thumb_io, format="JPEG")

        return File(thumb_io, name=self.original_filename)



# -*- coding: utf-8 -*-
from django.utils.text import slugify
from django.utils.translation import override
from django.test.testcases import TestCase
from django.conf import settings
from django.test.client import RequestFactory
from django.template import Template, RequestContext
from cms import api
from parler.utils.context import switch_language
from allink_core.core.test import PageApphookMixin, CategoriesMixin, DataModelMixin, AllinkAppContenPluginMixin
from allink_core.apps.news.cms_apps import NewsApphook
from allink_core.apps.config.utils import get_fallback
from allink_core.apps.config.tests.factories import ConfigFactory
from ..models import News
from ..cms_plugins import CMSNewsAppContentPlugin
from .factories import NewsFactory, NewsWithMetaFactory


class NewsTestCase(PageApphookMixin, DataModelMixin, TestCase):

    apphook = 'NewsApphook'
    namespace = 'news'
    page_template = 'default.html'

    apphook_object = NewsApphook
    data_model_factory = NewsFactory

    def test_create_instance(self):
        self.assertEqual(self.entry_1.status, News.ACTIVE)

    def test_delete_instance(self):
        News.objects.get(id=self.entry_1.id).delete()
        with self.assertRaises(News.DoesNotExist):
            News.objects.get(id=self.entry_1.id)

    def test___str__(self):
        self.assertEqual(self.entry_1.__str__(), '{} - {}'.format(
            self.entry_1.title,
            self.entry_1.entry_date.strftime('%d.%m.%Y %H:%M:%S')
        ))

    def test_slug(self):
        self.assertEqual(self.entry_1.slug, slugify(self.entry_1.title))

    def test_get_detail_view(self):
        self.assertEqual(self.entry_1.get_detail_view(), '{}:detail'.format(self.entry_1._meta.model_name))
        self.assertEqual(self.entry_1.get_detail_view('apphook_page'), 'apphook_page:detail')

    def test_get_absolute_url(self):
        self.assertEqual(self.entry_1.get_absolute_url(), '{}{}/'.format(
            self.page.get_absolute_url(), self.entry_1.slug))

    def test_get_absolute_url_no_apphook_configured(self):
        self.page.unpublish(language=self.language)
        self.assertEqual(self.entry_1.get_absolute_url(), '/no_apphook_configured')

    def test_get_absolute_url_specifc_application_namespace(self):
        other_page = api.create_page(
            'page', self.page_template,
            self.language,
            published=True,
            parent=self.root_page,
            apphook=self.apphook,
            apphook_namespace='some_application_namespace'
        )
        self.assertEqual(self.entry_1.get_absolute_url(application_namespace=other_page.application_namespace), '{}{}/'.format(
            other_page.get_absolute_url(), self.entry_1.slug))

    def test_get_absolute_url_language_fallback(self):
        # no translations
        # known_translation_getter returns 'en'
        with override('de'):
            self.assertEqual(self.entry_1.get_absolute_url(), '{}{}/'.format(
                self.page.get_absolute_url(language='en'), self.entry_1.safe_translation_getter('slug', language_code='en')))
            self.assertIn('/en/', self.entry_1.get_absolute_url())

    def test_get_absolute_url_language_translated(self):
        # translate entry in all languages
        for language, _ in settings.LANGUAGES[1:]:
            with switch_language(self.entry_1, language):
                self.entry_1.title = '{}: {}'.format(self.entry_1.title, language)
                self.entry_1.save()

        # known_translation_getter returns 'de'
        with override('de'):
            self.assertEqual(
                self.entry_1.get_absolute_url(),
                '{}{}/'.format(self.page.get_absolute_url(), self.entry_1.safe_translation_getter('slug', language_code='de'))
            )
            self.assertIn('/de/', self.entry_1.get_absolute_url())


class NewsMetaTestCase(TestCase):

    def setUp(self):
        self.entry_1 = NewsWithMetaFactory()
        self.allink_config = ConfigFactory()

    def render_template(self, string, request, context=None):
        context = context or {}
        context = RequestContext(request, context)
        context.request = request
        return Template(string).render(context)

    def test_get_page_title_with_appropriate_fallbacks(self):
        self.assertEqual(self.entry_1.meta_page_title, 'og title | default_base_title_en')

        self.entry_1.og_title = ''
        self.assertEqual(self.entry_1.meta_page_title, 'teaser title | default_base_title_en')

        self.entry_1.teaser_title = ''
        self.assertEqual(self.entry_1.meta_page_title, '{} | default_base_title_en'.format(self.entry_1.title))

        # remove default_base_title_en from allink_config
        self.allink_config.default_base_title = ''
        # also remove default translation 'en'
        with switch_language(self.allink_config, 'en'):
            self.allink_config.default_base_title = ''
            self.allink_config.save()
        self.allink_config.save()
        self.assertEqual(self.entry_1.meta_page_title, '{} | '.format(self.entry_1.title))

    def test_get_meta_title_with_appropriate_fallbacks(self):
        self.assertEqual(get_fallback(self.entry_1, 'meta_title'), 'og title')
        self.entry_1.og_title = ''
        self.assertEqual(get_fallback(self.entry_1, 'meta_title'), 'teaser title')
        self.entry_1.teaser_title = ''
        self.assertEqual(get_fallback(self.entry_1, 'meta_title'), self.entry_1.title)

    def test_get_meta_description_with_appropriate_fallbacks(self):
        self.assertEqual(get_fallback(self.entry_1, 'meta_description'), 'og description')
        self.entry_1.og_description = ''
        self.assertEqual(get_fallback(self.entry_1, 'meta_description'), 'teaser description')
        self.entry_1.teaser_description = ''
        self.assertEqual(get_fallback(self.entry_1, 'meta_description'), self.entry_1.lead)

    def test_get_meta_image_with_appropriate_fallbacks(self):
        self.assertEqual(get_fallback(self.entry_1, 'meta_image'), self.entry_1.og_image)
        self.entry_1.og_image = None
        self.assertEqual(get_fallback(self.entry_1, 'meta_image'), self.entry_1.teaser_image)
        self.entry_1.teaser_image = None
        self.assertEqual(get_fallback(self.entry_1, 'meta_image'), self.entry_1.preview_image)

    def test_get_meta_dict(self):
        expected_meta_context = {
            'meta_page_title': self.entry_1.meta_page_title,
            'meta_og_title': get_fallback(self.entry_1, 'meta_title'),
            'meta_description': get_fallback(self.entry_1, 'meta_description'),
            'meta_image_url': getattr(self.entry_1.meta_image_thumb, 'url', ''),
            'google_site_verification': self.allink_config.google_site_verification,
        }
        self.assertDictEqual(expected_meta_context, self.entry_1.meta_dict)

    def test_render_meta_og_templatetag_with_object_contains_correct_values(self):
        request = RequestFactory().get(self.entry_1.get_absolute_url())
        rendered_template = self.render_template(
            string='''
            {% load allink_meta_tags %}
            {% render_meta_og obj=obj %}''',
            request=request,
            context={'obj': self.entry_1}
        )
        self.assertIn(self.entry_1.meta_page_title, rendered_template)
        self.assertIn(get_fallback(self.entry_1, 'meta_title'), rendered_template)
        self.assertIn(get_fallback(self.entry_1, 'meta_description'), rendered_template)
        self.assertIn(get_fallback(self.entry_1, 'meta_image').url, rendered_template)
        self.assertIn('1200x630', rendered_template)
        self.assertIn(self.allink_config.google_site_verification, rendered_template)

    def test_rendered_template_with_object_contains_correct_overwitten_values(self):
        request = RequestFactory().get(self.entry_1.get_absolute_url())
        rendered_template = self.render_template(
            string='''
            {% load allink_meta_tags %}
            {% render_meta_og obj=obj overwrite_dict="{'meta_title': 'Hello', 'meta_description': 'Some Text', 'meta_image_url':'www.foo.ch/image.jpg'}" %}''',
            request=request,
            context={'obj': self.entry_1}
        )
        self.assertIn('Hello', rendered_template)
        self.assertIn(get_fallback(self.entry_1, 'meta_title'), rendered_template)
        self.assertIn('Some Text', rendered_template)
        self.assertIn('www.foo.ch/image.jpg', rendered_template)
        self.assertIn(self.allink_config.google_site_verification, rendered_template)


class NewsTeaserTestCase(TestCase):

    def setUp(self):
        self.entry_1 = NewsWithMetaFactory()
        self.allink_config = ConfigFactory()

    def test_teaser_title_with_appropriate_fallbacks(self):
        self.assertEqual(get_fallback(self.entry_1, 'teaser_title'), 'teaser title')

        self.entry_1.teaser_title = ''
        self.assertEqual(get_fallback(self.entry_1, 'teaser_title'), self.entry_1.title)

    def test_teaser_description_with_appropriate_fallbacks(self):
        self.assertEqual(get_fallback(self.entry_1, 'teaser_description'), 'teaser description')

        self.entry_1.teaser_description = ''
        self.assertEqual(get_fallback(self.entry_1, 'teaser_description'), self.entry_1.lead)

    def test_teaser_technical_title_with_appropriate_fallbacks(self):
        self.assertEqual(get_fallback(self.entry_1, 'teaser_technical_title'), 'teaser technical title')

        self.entry_1.teaser_technical_title = ''
        self.assertEqual(get_fallback(self.entry_1, 'teaser_technical_title'), self.entry_1.entry_date)

    def test_teaser_link_text_with_appropriate_fallbacks(self):
        self.assertEqual(get_fallback(self.entry_1, 'teaser_link_text'), 'teaser link text')

        self.entry_1.teaser_link_text = ''
        self.assertEqual(get_fallback(self.entry_1, 'teaser_link_text'), 'Read more')

    def test_teaser_image_with_appropriate_fallbacks(self):
        self.assertEqual(get_fallback(self.entry_1, 'teaser_image'), self.entry_1.teaser_image)

        self.entry_1.teaser_image = None
        self.assertEqual(get_fallback(self.entry_1, 'teaser_image'), self.entry_1.preview_image)

    def test_teaser_dict(self):

        expected_meta_context = {
            'teaser_title': get_fallback(self.entry_1, 'teaser_title'),
            'teaser_technical_title': get_fallback(self.entry_1, 'teaser_technical_title'),
            'teaser_description': get_fallback(self.entry_1, 'teaser_description'),
            'teaser_image': get_fallback(self.entry_1, 'teaser_image'),
            'teaser_link_text': get_fallback(self.entry_1, 'teaser_link_text'),
            'teaser_link': self.entry_1.get_absolute_url(),
        }
        self.assertDictEqual(expected_meta_context, self.entry_1.teaser_dict)


class NewsPluginTestCaseApp(CategoriesMixin, DataModelMixin, AllinkAppContenPluginMixin, TestCase):
    apphook = 'NewsApphook'
    namespace = 'news'
    page_template = 'default.html'
    apphook_object = NewsApphook
    data_model_factory = NewsFactory
    plugin_class = CMSNewsAppContentPlugin

    def test_no_filter_no_manual_select(self):
        self.assertEqual(self.plugin_model_instance.get_render_queryset_for_display().count(), 5)

    def test_no_filter_manual_select(self):
        self.plugin_model_instance.manual_entries.set([self.entry_1, self.entry_2])
        self.assertEqual(self.plugin_model_instance.get_render_queryset_for_display().count(), 2)

    def test_no_filter_manual_select_filter_category(self):
        self.plugin_model_instance.manual_entries.set([self.entry_1, self.entry_2, self.entry_3])

        self.entry_1.categories.set([self.category_1, self.category_2])
        self.entry_2.categories.set([self.category_1, self.category_2])

        self.assertEqual(self.plugin_model_instance.get_render_queryset_for_display().count(), 3)
        self.assertEqual(self.plugin_model_instance.get_render_queryset_for_display(category=self.category_1).count(), 2)

    def test_filter_category(self):
        self.plugin_model_instance.categories.set([self.category_1])

        self.entry_1.categories.set([self.category_1, self.category_2])
        self.entry_2.categories.set([self.category_1, self.category_2])

        self.assertEqual(self.plugin_model_instance.get_render_queryset_for_display().count(), 2)
        self.assertEqual(self.plugin_model_instance.get_render_queryset_for_display(category=self.category_1).count(), 2)

    def test_filter_category_multiple_categories_set(self):
        self.plugin_model_instance.categories.set([self.category_1, self.category_2])

        self.entry_1.categories.set([self.category_1, self.category_2])
        self.entry_2.categories.set([self.category_1, self.category_2])
        self.entry_3.categories.set([self.category_2])

        self.assertEqual(self.plugin_model_instance.get_render_queryset_for_display().count(), 3)
        self.assertEqual(self.plugin_model_instance.get_render_queryset_for_display(category=self.category_1).count(), 2)
        self.assertEqual(self.plugin_model_instance.get_render_queryset_for_display(category=self.category_2).count(), 3)

    def test_filter_filters_categories_and__in_no_result(self):
        self.plugin_model_instance.categories.set([self.category_1])
        self.plugin_model_instance.categories_and.set([self.category_2])

        self.entry_1.categories.set([self.category_1])
        self.entry_2.categories.set([self.category_2])
        self.assertEqual(self.plugin_model_instance.get_render_queryset_for_display().count(), 0)

    def test_filter_filters_categories_and__in_result(self):
        self.plugin_model_instance.categories.set([self.category_1])
        self.plugin_model_instance.categories_and.set([self.category_2])

        self.entry_1.categories.set([self.category_1])
        self.entry_2.categories.set([self.category_2])
        self.entry_3.categories.set([self.category_1, self.category_2])
        self.assertEqual(self.plugin_model_instance.get_render_queryset_for_display().count(), 1)
        self.assertEqual(self.plugin_model_instance.get_render_queryset_for_display().first().id, self.entry_3.id)





from import_export import resources, fields

from allink_core.core_apps.allink_legacy_redirect.models import AllinkLegacyLink


class AllinkLegacyLinkResource(resources.ModelResource):
    old = fields.Field(column_name='target', attribute='old')

    class Meta:
        model = AllinkLegacyLink
        fields = ['old']
        import_id_fields = ['old']
        skip_unchanged = True

    def skip_row(self, instance, original):
        out = super(AllinkLegacyLinkResource, self).skip_row(instance, original)
        # skip urls which we never want to be redirected
        if not out and instance.old in ['(not set)','/']:
            return True
        return out



# Generated by Django 2.2.7 on 2020-02-05 14:36

from django.db import migrations

def set_new_default_value(apps, schema_editor):
    AllinkTeaserPlugin = apps.get_model('allink_teaser', 'AllinkTeaserPlugin')
    for teaser in AllinkTeaserPlugin.objects.all():
        teaser.softpage_enabled = False
        teaser.save()


class Migration(migrations.Migration):

    dependencies = [
        ('allink_teaser', '0005_auto_20200205_1427'),
    ]

    operations = [
        migrations.RunPython(set_new_default_value),
    ]



# -*- coding: utf-8 -*-
import mandrill
from raven import Client
from django.conf import settings
from django.utils.translation import get_language
from .config import MandrillConfig


def check_result_status(result):
    if result[0].get('status') != 'sent' and result[0].get('status') != 'queued':
        raise mandrill.Error(
            "Mandrill hasn't raised an error but email could not been sent. (status: '{}', reason: '{}')".format(
                result[0].get('status'),
                result[0].get('reject_reason')
            ))


# backwards compat
def send_transactional_email(message, template_content, language=None, translated=False,
                             template_name=None, async=False):
    config = MandrillConfig()

    if not template_name:
        template_name = config.default_transactional_template_name
    if not language and translated:
        template_name = '{}_{}'.format(template_name, get_language())
    if language:
        template_name = '{}_{}'.format(template_name, language)

    try:
        mandrill_client = mandrill.Mandrill(getattr(settings, 'MANDRILL_API_KEY'))
        result = mandrill_client.messages.send_template(
            template_name=template_name,
            template_content=template_content,
            message=message,
            async=async
        )
        check_result_status(result)
    except:
        client = Client(settings.RAVEN_CONFIG.get('dsn'))
        client.captureException()
        # sentry is not configured on localhost
        if not settings.RAVEN_CONFIG.get('dsn'):
            raise



# -*- coding: utf-8 -*-

from django.db import models
from django.utils.encoding import python_2_unicode_compatible
from cms.models.pluginmodel import CMSPlugin


@python_2_unicode_compatible
class AllinkSEOAccordionContainerPlugin(CMSPlugin):
    """
    A Container-Plugin for SEO Accordion content
    """

    is_seo_faq = models.BooleanField(
        'Enable SEO FAQ schema',
        help_text='Enable to display accordion contents as questions/answers in search engine result pages',
        default=True
    )

    def __str__(self):
        return '{}'.format(str(self.pk))


@python_2_unicode_compatible
class AllinkSEOAccordion(CMSPlugin):
    title = models.CharField(
        'Title',
        max_length=255
    )

    def __str__(self):
        return '{}'.format(self.title)



# -*- coding: utf-8 -*-
import json
from django.views.generic import ListView, DetailView, FormView
from django.db.models import Q
from django.shortcuts import render
from django.utils.translation import ugettext_lazy as _
from django.urls import reverse
from django.core.exceptions import ImproperlyConfigured
from django.http import HttpResponse, HttpResponsePermanentRedirect, Http404, JsonResponse
from django.template.loader import render_to_string
from parler.views import TranslatableSlugMixin

from allink_core.core.models import AllinkBaseAppContentPlugin
from allink_core.core_apps.allink_categories.models import AllinkCategory
from allink_core.core.utils import update_context_google_tag_manager


class AllinkBasePluginLoadMoreView(ListView):

    plugin_model = None

    def get_queryset(self):
        if hasattr(self, 'category'):
            return self.plugin.get_render_queryset_for_display(category=self.category)
        else:
            return self.plugin.get_render_queryset_for_display()

    def get_paginate_by(self, queryset):
        if self.plugin.paginated_by != 0:
            return self.plugin.paginated_by
        else:
            return None

    def get_context_data(self, **kwargs):
        context = {
            'request': self.request,
            'instance': self.plugin,
            'appended': True
        }
        queryset = kwargs.pop('object_list', self.object_list)
        page_size = self.get_paginate_by(queryset)

        if page_size:
            paginator, page, queryset, is_paginated = self.paginate_queryset(queryset, page_size)
            if self.plugin.pagination_type == AllinkBaseAppContentPlugin.LOAD_REST:
                # !! WARNING: the paginator and page_obj are not set correctly. Only the queryset is correct.
                # Yeah, not so pretty. But we couldn't find a way to alternate page_size with the django paginator.
                # this would probably be better moved to a separate view. (e.g AllinkBasePluginLoadRestView, which
                # takes the plugin id, a list of ids to be excluded from the queryset.)

                # exclude the first page from the full queryset
                queryset = self.get_queryset().filter(~Q(id__in=[o.id for o in paginator.page(1).object_list]))
                page = paginator.page(paginator.num_pages)  # fake the page to be the last page
            context.update({
                'paginator': paginator,
                'page_obj': page,
                'is_paginated': is_paginated,
                'object_list': queryset
            })
        else:
            context.update({
                'paginator': None,
                'page_obj': None,
                'is_paginated': False,
                'object_list': queryset
            })
        context.update(kwargs)
        return context

    def get_template_names(self, context):
        file = 'no_results' if not context['object_list'] else '_items'
        return [self.plugin.get_correct_template(file)]

    def get(self, request, *args, **kwargs):
        try:
            self.plugin = self.plugin_model.objects.get(cmsplugin_ptr_id=request.GET.get('plugin_id'))
            if 'category' in request.GET.keys():
                self.category_id = request.GET.get('category', None)
                self.category = AllinkCategory.objects.get(id=self.category_id)

        except (KeyError, self.plugin_model.DoesNotExist) as e:
            # this view is useless when we do not ha a plugin_id
            # or a plugin instance where we can get our information from
            raise Http404(e)

        self.object_list = self.get_queryset()
        context = self.get_context_data()
        return self.json_response(context)

    def json_response(self, context):
        json_context = {}

        json_context['rendered_content'] = render_to_string(
            self.get_template_names(context)[0], context=context,
            request=self.request
        )

        # no need to create next_page_url when no pagination should be displayed
        if self.plugin.paginated_by > 0:

            if context['page_obj'].has_next():
                get_params = '&'.join(['%s=%s' % (k, v) for k, v in self.request.GET.items()])

                json_context['next_page_url'] = reverse(
                    '{}:more'.format(self.plugin.data_model._meta.model_name),
                    kwargs={'page': context['page_obj'].next_page_number()}
                ) + '?{}'.format(get_params)

                json_context['next_page_url'] = json_context['next_page_url'] + '&category={}'.format(
                    self.category_id) if hasattr(self, 'category_id') else json_context['next_page_url']
        json_context['no_results'] = False if self.object_list else True
        return HttpResponse(content=json.dumps(json_context), content_type='application/json', status=200)


class AllinkBaseDetailView(TranslatableSlugMixin, DetailView):
    """
    model = Events
    """

    def get_template_names(self):
        obj = self.get_object()
        context = self.get_context_data(object=obj)
        categories = context['object'].categories.all()
        app_label = context['object']._meta.app_label
        templates = ['{}/{}_detail.html'.format(app_label, self.request.current_page.application_namespace)]
        templates += ['{}/{}_{}_detail.html'.format(app_label, app_label, category.slug) for category in categories]
        templates += super().get_template_names()
        return templates

    def get(self, request, *args, **kwargs):
        self.object = self.get_object()
        if not request.user.is_staff and self.object.status == self.object.INACTIVE:
            raise Http404(_("The requested content is not published.") % {'class_name': self.__class__.__name__})
        context = self.get_context_data(object=self.object)

        return self.render_to_response(context)

    def render_to_response(self, context, **response_kwargs):
        if self.request.GET.get('softpage', None):
            context.update({'base_template': 'app_content/ajax_base.html'})
        return render(self.request, self.get_template_names(), context)


class AllinkBasePluginAjaxFormView(FormView):
    """
    Use this class whenever you create a plugin which displays a form.

    example implementation:

    class ProductTrackingView(AllinkBasePluginAjaxFormView):
        form_class = ProductTrackingForm
        template_name = 'product/plugins/tracking/content.html'

        success_template_name = 'product/plugins/tracking/success.html'
        plugin_model = ProductTrackingPlugin

    example urls.py:
    path('tracking/(<int:plugin_id>/', ProductTrackingView.as_view(), name='tracking'),

    to ensure that search engines do not index the view when called with GET (unstyled),
    we only allow POST (the GET is with the cmsplugin render)
    http_method_names = ['post']

    therefore views inherit from this class are not suitable for direct links
    e.g all viewnames listed in BUTTON_LINK_SPECIAL_LINKS_CHOICES.
    these views will be called by a GET request and not with ajax
    """

    http_method_names = ['post']

    def post(self, request, plugin_id, *args, **kwargs):
        """
        Handles POST requests, instantiating a form instance with the passed
        POST variables and then checked for validity.

        adds the plugin instance from the plugin_id kwargs
        """
        self.plugin_instance = self.plugin_model.objects.get(id=plugin_id)
        _, self.plugin = self.plugin_instance.get_plugin_instance()

        return super().post(request, *args, **kwargs)

    def get_context_data(self, *args, **kwargs):
        """
        Add the plugin instance context, so it ca be rendered with all necessary data
        and the same plugin template used by the cms
        """
        context = super().get_context_data(*args, **kwargs)
        context.update({
            'csrf_token_value': self.request.COOKIES.get('csrftoken'),
            'request': self.request,
            'instance': self.plugin_instance,
            'action': self.plugin.get_form_action(self.plugin_instance),
        })
        # to be removed? because we could actually track the form with ""
        context = update_context_google_tag_manager(
            context=context,
            page_name=self.request.current_page.__str__(),
            page_id=self.request.current_page.id,
            plugin_id=self.plugin_instance,
            name=self.__class__.__name__
        )
        return context

    def form_valid(self, form):
        """
        We return the response as json, with status code 200
        """
        context = self.get_context_data(form=form)
        return self.get_json_response(
            context=context,
            status=200,
            template_name=self.get_success_template_names()[0]
        )

    def form_invalid(self, form):
        """
        We return the response as json, with status code 206
        """
        return self.get_json_response(
            context=self.get_context_data(form=form),
            status=206,
            template_name=self.get_template_names()[0]
        )

    def get_success_template_names(self):
        """
        Returns a list of template names to be used for the successfull request. Must return
        a list. May not be called if form_valid is overridden.
        """
        if self.success_template_name is None:
            raise ImproperlyConfigured(
                "AllinkBasePluginAjaxFormView requires either a definition of "
                "'success_template_name' or an implementation of 'get_success_template_names()'")
        else:
            return [self.success_template_name]

    def get_json_response(self, context, status, template_name=None, data=None):
        """
        This response will most likely be handled by allink-core-static/js/modules/ajax-form.js

        context: the context which will we passed to template_name
        template_name: the template to render with
        status: 200 or 206, most likely
        data: additional response data (e.g 'success_url')

        * there is no template needed, when we redirect to a success_url
        """
        if not data:
            data = dict()

        if template_name:
            data.update({
                'rendered_content': render_to_string(template_name, context=context, request=self.request)
            })
        return JsonResponse(data, status=status)


class AllinkBasePluginAjaxCreateView(AllinkBasePluginAjaxFormView):
    """
    Use this class whenever you create a plugin which displays a form.

    example implementation:

    class ProductTrackingView(AllinkBasePluginAjaxCreateView):
        model = ProductTracking
        form_class = ProductTrackingForm
        template_name = 'product/plugins/tracking/content.html'

        success_template_name = 'product/plugins/tracking/success.html'
        plugin_model = ProductTrackingPlugin

    example urls.py:
    path('tracking/(<int:plugin_id>/', ProductTrackingView.as_view(), name='tracking'),

    to ensure that search engines do not index the view when called with GET (unstyled),
    we only allow POST (the GET is with the cmsplugin render)
    http_method_names = ['post']

    therefore views inherit from this class are not suitable for direct links
    e.g all viewnames listed in BUTTON_LINK_SPECIAL_LINKS_CHOICES.
    these views will be called by a GET request and not with ajax
    """

    def form_valid(self, form):
        """If the form is valid, save the associated model."""
        self.object = form.save()
        return super().form_valid(form)


# used to redirect CMSPage to external url
def external_view(request, target):
    return HttpResponsePermanentRedirect('http://' + target)



# -*- coding: utf-8 -*-

__all__ = [
    'ALLINK_INSTALLED_APPS',
    'ALLINK_PROJECT_APP_MODEL_WITH_CATEGORY_CHOICES',
    'ALLINK_CMS_PLACEHOLDER_CONF_PLUGINS',
    'ALLINK_LOCALE_PATHS',
    'ALLINK_PAGE_TOOLBAR_ENABLED',
    'ALLINK_MANDRILL_DEV_MODE',
    'ALLINK_MANDRILL_DEV_MODE_FROM_EMAIL_ADDRESS',
    'ALLINK_MANDRILL_DEV_MODE_TO_EMAIL_ADDRESSES',
    'ALLINK_CONTENT_PLUGIN_CHILD_CLASSES',
    'ALLINK_SEOACCORDION_PLUGIN_CHILD_CLASSES',
    'THUMBNAIL_QUALITY',
    'THUMBNAIL_OPTIMIZE_COMMAND',
    'DEBUG_TOOLBAR_CONFIG',
    'SOLO_CACHE_TIMEOUT',
]

####################################################################################

# Installed =Apps

ALLINK_INSTALLED_APPS = [
    # apps
    'debug_toolbar',
    'webpack_loader',
    'adminsortable',
    'sortedm2m',
    'solo',
    'import_export',
    'widget_tweaks',
    'lockdown',
    'reportlab',
    'djangocms_snippet',
    'djangocms_attributes_field',
    'django_countries',
    'django.contrib.postgres',
    'inline_static',

    # allink core apps
    'allink_core.core',
    'allink_core.core_apps.allink_categories',
    'allink_core.core_apps.allink_styleguide',
    'allink_core.core_apps.allink_legacy_redirect',

    'allink_core.core_apps.allink_quote',
    'allink_core.core_apps.allink_content',
    'allink_core.core_apps.allink_teaser',
    'allink_core.core_apps.allink_image',
    'allink_core.core_apps.allink_gallery',
    'allink_core.core_apps.allink_video',
    'allink_core.core_apps.allink_social_icon',
    'allink_core.core_apps.allink_button_link',
    'allink_core.core_apps.allink_cms',
    # 'allink_core.core_apps.allink_info_box',
    'allink_core.core_apps.allink_seo_accordion',
]

####################################################################################

# =allink categories
# all models which use categories have to be listed here.
# the value has to be equal to "_meta.model_name"

ALLINK_PROJECT_APP_MODEL_WITH_CATEGORY_CHOICES = [
    ('people', 'People'),
    ('news', 'News'),
    # ('locations', 'Locations'),
]

####################################################################################

# Assign Plugins to =Placeholders

# http://docs.django-cms.org/en/develop/reference/configuration.html#cms-placeholder-conf

ALLINK_CMS_PLACEHOLDER_CONF_PLUGINS = [
    'CMSAllinkContentPlugin',
    'Module',
    # 'CMSAllinkInfoBoxPlugin',
]

####################################################################################

# Assign Plugins which are allowed as child in Allink Content

ALLINK_CONTENT_PLUGIN_CHILD_CLASSES = [
    'TextPlugin',
    'SnippetPlugin',
    'AliasPlugin',
    # apps
    'CMSLocationsAppContentPlugin',
    'CMSPeopleAppContentPlugin',
    'CMSNewsAppContentPlugin',
    # core
    'CMSAllinkTeaserPlugin',
    'CMSAllinkImagePlugin',
    'CMSAllinkGalleryPlugin',
    'CMSAllinkVideoEmbedPlugin',
    'CMSAllinkVideoFilePlugin',
    'CMSAllinkSocialIconContainerPlugin',
    'CMSAllinkButtonLinkContainerPlugin',
    # 'CMSAllinkPageChooserPlugin',
    'CMSAllinkLanguageChooserPlugin',
    'CMSAllinkSEOAccordionContainerPlugin',
    'CMSAllinkQuotePlugin',
]

####################################################################################

# Assign Plugins which are allowed as child in Allink SEO Accordion Content

ALLINK_SEOACCORDION_PLUGIN_CHILD_CLASSES = [
    'TextPlugin',
]

####################################################################################

# Locale

ALLINK_LOCALE_PATHS = [
    # project
    '/app/locale_extra',
    '/app/locale',
    # apps
    '/app/allink_core/apps/news/locale',
    '/app/allink_core/apps/config/locale',
    '/app/allink_core/apps/locations/locale',
    '/app/allink_core/apps/people/locale',
    # core
    '/app/allink_core/core/locale',
    # core_apps
    '/app/allink_core/core_apps/allink_categories/locale',
    '/app/allink_core/core_apps/allink_legacy_redirect/locale',
    '/app/allink_core/core_apps/allink_mandrill/locale',
    '/app/allink_core/core_apps/allink_styleguide/locale',
    '/app/allink_core/core_apps/allink_button_link/locale',
    '/app/allink_core/core_apps/allink_content/locale',
    '/app/allink_core/core_apps/allink_gallery/locale',
    '/app/allink_core/core_apps/allink_image/locale',
    '/app/allink_core/core_apps/allink_social_icon/locale',
    '/app/allink_core/core_apps/allink_video/locale',
    '/app/allink_core/core_apps/allink_cms/locale',
]

# ####################################################################################
#
# # Thumbnail

THUMBNAIL_QUALITY = 60

THUMBNAIL_OPTIMIZE_COMMAND = {
    'png': 'optipng {filename}',
    'gif': 'optipng {filename}',
    'jpeg': 'jpegoptim {filename}'
}


#  ####################################################################################
#
# # Debug Toolbar


def show_toolbar(request):
    return False


DEBUG_TOOLBAR_CONFIG = {
    "SHOW_TOOLBAR_CALLBACK": show_toolbar,
}

#  ####################################################################################
#
# # allink Page Extension

ALLINK_PAGE_TOOLBAR_ENABLED = False

#  ####################################################################################
#
# # allink E-Mail Mandrill
# if ALLINK_MANDRILL_DEV_MODE is set to True all emails will be sent to ALLINK_MANDRILL_DEV_MODE_TO_EMAIL_ADDRESSES
ALLINK_MANDRILL_DEV_MODE = True
ALLINK_MANDRILL_DEV_MODE_FROM_EMAIL_ADDRESS = 'test@allink.ch'
# we cant send to 'test@allink.ch' at the moment because of: https://support.google.com/a/answer/168383?hl=en
ALLINK_MANDRILL_DEV_MODE_TO_EMAIL_ADDRESSES = ['itcrowd@allink.ch', ]

#  ####################################################################################
#
# # cache
SOLO_CACHE_TIMEOUT = 60 * 60 * 24 * 180



# -*- coding: utf-8 -*-
from __future__ import unicode_literals, absolute_import

from django import forms

from allink_core.core.models import choices
from allink_core.core_apps.allink_button_link import widgets


class Context(forms.fields.ChoiceField):
    widget = widgets.Context
    CHOICES = choices.CONTEXT_CHOICES
    DEFAULT = choices.CONTEXT_DEFAULT

    def __init__(self, *args, **kwargs):
        if 'choices' not in kwargs:
            kwargs['choices'] = self.CHOICES
        if 'initial' not in kwargs:
            kwargs['initial'] = self.DEFAULT
        kwargs.pop('coerce', None)
        kwargs.pop('max_length', None)
        kwargs.pop('widget', None)
        kwargs['widget'] = self.widget
        super(Context, self).__init__(*args, **kwargs)


class Size(forms.fields.ChoiceField):
    widget = widgets.Size
    CHOICES = choices.SIZE_CHOICES
    DEFAULT = 'md'

    def __init__(self, *args, **kwargs):
        if 'choices' not in kwargs:
            kwargs['choices'] = self.CHOICES
        if 'initial' not in kwargs:
            kwargs['initial'] = self.DEFAULT
        kwargs.pop('coerce', None)
        kwargs.pop('max_length', None)
        kwargs.pop('widget', None)
        kwargs['widget'] = self.widget
        super(Size, self).__init__(*args, **kwargs)


class LinkOrButton(forms.fields.ChoiceField):
    widget = widgets.LinkOrButton
    CHOICES = (
        ('lnk', 'Link'),
        ('btn', 'Button'),
    )
    DEFAULT = 'lnk'

    def __init__(self, *args, **kwargs):
        if 'choices' not in kwargs:
            kwargs['choices'] = self.CHOICES
        if 'initial' not in kwargs:
            kwargs['initial'] = self.DEFAULT
        kwargs.pop('coerce', None)
        kwargs.pop('max_length', None)
        kwargs.pop('widget', None)
        kwargs['widget'] = self.widget
        super(LinkOrButton, self).__init__(*args, **kwargs)



# -*- coding: utf-8 -*-
from django import forms
from django.core.paginator import Paginator
from django.urls import reverse
from django.contrib.admin.widgets import FilteredSelectMultiple

from cms.plugin_base import CMSPluginBase

from allink_core.core.models import AllinkBaseAppContentPlugin, AllinkBaseSearchPlugin
from allink_core.core.utils import get_project_css_classes
from allink_core.core.admin.mixins import AllinkMediaAdminMixin


class AllinkBaseAppContentPluginForm(forms.ModelForm):
    class Meta:
        model = AllinkBaseAppContentPlugin
        exclude = ()

    def __init__(self, *args, **kwargs):
        super(AllinkBaseAppContentPluginForm, self).__init__(*args, **kwargs)
        # if app uses categories, populate 'categories' field
        if self.instance.get_app_can_have_categories():
            self.fields['categories'] = forms.ModelMultipleChoiceField(
                label='Categories',
                widget=FilteredSelectMultiple(
                    verbose_name='Categories',
                    is_stacked=True
                ),
                required=False,
                queryset=self.instance.data_model.get_relevant_categories()
            )
            self.fields['categories_and'] = forms.ModelMultipleChoiceField(
                label='Categories (AND operator)',
                widget=FilteredSelectMultiple(
                    verbose_name='Categories',
                    is_stacked=True
                ),
                help_text=(
                    'Use this field if you want to further restrict your result set. This option allows you to create'
                    ' a conjunction between the first set of categories in field "Categories" and the ones '
                    'specified here.'),
                required=False,
                queryset=self.instance.data_model.get_relevant_categories()
            )
            self.fields['category_navigation'] = forms.ModelMultipleChoiceField(
                label='Categories for Navigation',
                widget=FilteredSelectMultiple(
                    verbose_name='Categories for Navigation',
                    is_stacked=True
                ),
                help_text=(
                    'You can explicitly define the categories for the category navigation here. '
                    'This will override the'
                    ' automatically set of categories. (From "Filter & Ordering" but not from the "Manual entries")'),
                required=False,
                queryset=self.instance.data_model.get_relevant_categories()
            )

        self.fields['template'] = forms.CharField(
            label='Template',
            widget=forms.Select(choices=self.instance.get_templates()),
            required=True,
        )
        if get_project_css_classes(self._meta.model.data_model._meta.model_name):
            self.fields['project_css_classes'] = forms.MultipleChoiceField(
                widget=forms.CheckboxSelectMultiple(),
                label='Predifined variations',
                choices=get_project_css_classes(self._meta.model.data_model._meta.model_name),
                required=False,
            )
        self.fields['manual_filtering'] = forms.CharField(
            label='Filtering',
            required=False,
            widget=forms.Select(choices=self.instance.get_filtering_choices() if hasattr(
                self.instance, 'get_filtering_choices') else [])
        )
        self.fields['manual_ordering'] = forms.CharField(
            label='Ordering',
            required=False,
            widget=forms.Select(choices=self.instance.get_ordering_choices())
        )


class CMSAllinkBaseAppContentPlugin(AllinkMediaAdminMixin, CMSPluginBase):
    """
    is not registered itself
    only used to inherit from (for specific App Content Plugins)
    """
    model = AllinkBaseAppContentPlugin

    name = 'App Content'
    module = 'allink modules'
    allow_children = False
    form = AllinkBaseAppContentPluginForm

    def get_fieldsets(self, request, obj=None):
        fieldsets = (
            (None, {
                'fields': [
                    'template',
                ],
            }),
        )

        fieldsets += ('Display Options', {
            'classes': ('collapse',),
            'fields': (
                'detail_link_enabled',
                'softpage_enabled',
            )
        }),

        if self.model.data_model.get_can_have_categories():
            fieldsets += ('Categories', {
                'classes': ('collapse',),
                'fields': (
                    'categories',
                    'categories_and',
                )
            }),

        fieldsets += ('Filter & Ordering', {
            'classes': ('collapse',),
            'fields': (
                'manual_filtering',
                'manual_ordering',
            )
        }),

        fieldsets += ('Select entries manually', {
            'classes': ('collapse',),
            'fields': (
                'manual_entries',
            )
        }),

        if self.model.data_model.get_can_have_categories():
            fieldsets += ('Category Navigation Options', {
                'classes': (
                    'collapse',
                    'disable_when_slider',
                    'disable_when_map',
                ),
                'fields': (
                    'category_navigation_enabled',
                    'category_navigation_all',
                    'category_navigation',
                )
            }),

        fieldsets += ('Number of entries', {
            'classes': (
                'collapse',
                'disable_when_map',
            ),
            'fields': (
                ('paginated_by',),
            )
        }),

        fieldsets += ('Pagination Options', {
            'classes': (
                'collapse',
                'disable_when_slider',
                'disable_when_map',
            ),
            'fields': (
                ('pagination_type', 'load_more_button_text'),
            )
        }),

        fieldsets += ('Additional Options', {
            'classes': ('collapse',),
            'fields': (
                'apphook_page',
                'detail_link_text',
                'project_css_classes',
            )
        }),

        fieldsets += ('Grid Options', {
            'classes': (
                'collapse',
                'only_when_grid_static',
                'only_when_grid_static_newsletter',
                'only_when_grid_dynamic',
            ),
            'fields': (
                'items_per_row',
            )
        }),

        return fieldsets

    def get_application_namespace(self, instance):
        if getattr(instance, 'apphook_page'):
            return instance.apphook_page.application_namespace
        else:
            return self.model.data_model._meta.model_name

    def get_render_template(self, context, instance, placeholder):
        file = 'no_results' if not context['object_list'] else 'content'
        return instance.get_correct_template(file)

    def get_queryset_by_category(self, instance):
        # category navigation and no category "all" (only first category is relevant)
        if instance.category_navigation_enabled and not instance.category_navigation_all:
            object_list = instance.get_render_queryset_for_display(category=instance.fetch_first_category)
        else:
            object_list = instance.get_render_queryset_for_display()
        return object_list

    def render(self, context, instance, placeholder):

        object_list = self.get_queryset_by_category(instance)

        # Paginate Objects
        if instance.paginated_by > 0:
            paginator = Paginator(object_list, instance.paginated_by)
            firstpage = paginator.page(1)
            object_list = firstpage.object_list

            # Load More
            if (instance.pagination_type == AllinkBaseAppContentPlugin.LOAD
                or instance.pagination_type == AllinkBaseAppContentPlugin.LOAD_REST) \
                    and firstpage.has_next():
                context['page_obj'] = firstpage
                context.update(
                    {'next_page_url': reverse('{}:more'.format(
                        self.get_application_namespace(instance)), kwargs={
                        'page': context['page_obj'].next_page_number()}) + '?plugin_id={}'.format(instance.id)})

                if instance.category_navigation_enabled and not instance.category_navigation_all:
                    context['next_page_url'] = \
                        context['next_page_url'] + '&category={}'.format(instance.fetch_category_navigation[0].id)

        # category navigation
        if instance.category_navigation_enabled:
            context.update(
                {'by_category': reverse('{}:more'.format(self.get_application_namespace(instance)),
                                        kwargs={'page': 1}) + '?plugin_id={}'.format(instance.id)})

        context['instance'] = instance
        context['placeholder'] = placeholder
        context['object_list'] = object_list

        if instance.category_navigation_enabled:
            context['category_navigation'] = instance.fetch_category_navigation

        return context


class AllinkBaseSearchPluginForm(forms.ModelForm):
    class Meta:
        model = AllinkBaseAppContentPlugin
        exclude = ()

    def __init__(self, *args, **kwargs):
        super(AllinkBaseSearchPluginForm, self).__init__(*args, **kwargs)
        if get_project_css_classes(self._meta.model.data_model._meta.model_name):
            self.fields['project_css_classes'] = forms.MultipleChoiceField(
                widget=forms.CheckboxSelectMultiple(),
                label='Predifined variations',
                choices=get_project_css_classes(self._meta.model.data_model._meta.model_name),
                required=False,
            )

        self.fields['template'] = forms.CharField(
            label='Template',
            widget=forms.Select(choices=self.instance.get_templates()),
            required=True,
        )


class CMSAllinkBaseSearchPlugin(AllinkMediaAdminMixin, CMSPluginBase):
    """
    is not registered itself
    only used to inherit from (for specific Search Plugins)
    """
    model = AllinkBaseSearchPlugin
    module = 'allink search'
    form = AllinkBaseSearchPluginForm
    search_form = None

    def get_render_template(self, context, instance, placeholder):
        return '{}/plugins/{}/content.html'.format(self.model.data_model._meta.app_label, instance.template)

    def render(self, context, instance, placeholder):
        context['instance'] = instance
        context['placeholder'] = placeholder

        form = self.search_form()
        object_list = self.model.data_model.objects.active()

        additional_context = [
            ('form', form),
            ('object_list', object_list),
        ]

        for key, val in additional_context:
            context.update({key: val})

        return context

    def get_fieldsets(self, request, obj=None):
        fieldsets = (
            (None, {
                'fields': [
                    'project_css_classes',
                    'template'
                ],
            }),
        )
        return fieldsets


class CMSAllinkBaseFormPlugin(CMSPluginBase):
    """
    Use this BasePlugin for plugins, which display a form.

    example implementation:

    class CMSLivingSignupPlugin(CMSAllinkBaseFormPlugin):
        name = 'Living Signup Plugin'
        model = LivingSignupPlugin
        render_template = 'living/plugins/signup/content.html'

        form_class = LivingSignupForm
        url_name = 'signup'

    """

    form_class = None
    url_name = None

    def render(self, context, instance, placeholder):
        context = super().render(context, instance, placeholder)

        context.update({
            'form': self.form_class(),
            'action': self.get_form_action(instance),
        })
        return context

    @property
    def view_name(self):
        """
        a string with the default application namespace (defined in cms_apps.py) and the url_name
        e.g 'living:signup'
        """
        return '{}:{}'.format(self.model._meta.app_label, self.url_name)

    def get_form_action(self, instance):
        """ reversed url. For the view, to send the form data to. """
        return reverse(self.view_name, kwargs={'plugin_id': instance.id})



# -*- coding: utf-8 -*-
from cms.models import CMSPlugin
from cms.plugin_base import CMSPluginBase
from cms.plugin_pool import plugin_pool
from djangocms_text_ckeditor.cms_plugins import TextPlugin, Text
from django.conf import settings

from allink_core.core_apps.allink_seo_accordion.models import AllinkSEOAccordionContainerPlugin, AllinkSEOAccordion


@plugin_pool.register_plugin
class CMSAllinkSEOAccordionContainerPlugin(CMSPluginBase):
    model = AllinkSEOAccordionContainerPlugin
    name = 'SEO Accordion Container'
    module = 'Generic'
    allow_children = True
    child_classes = ['CMSAllinkSEOAccordionPlugin']

    def get_fieldsets(self, request, obj=None):
        fieldsets = (
            (None, {
                'fields': (
                    'is_seo_faq',
                ),
            }),
        )

        return fieldsets

    def get_render_template(self, context, instance, placeholder):
        template = 'allink_seo_accordion/content.html'
        return template

    def save_model(self, request, obj, form, change):
        # Create 3 seo accordion entries on plugin creation with text plugin inside
        response = super().save_model(request, obj, form, change)
        if obj.numchild == 0:
            column_amount = 3

            for x in range(column_amount):
                plugin = AllinkSEOAccordion(
                    parent=obj,
                    placeholder=obj.placeholder,
                    language=obj.language,
                    position=CMSPlugin.objects.filter(parent=obj).count(),
                    plugin_type=CMSAllinkSEOAccordionPlugin.__name__
                )
                plugin.title = 'Title {}'.format(x + 1)
                plugin.save()

                child_plugin = Text(
                    parent=plugin,
                    placeholder=plugin.placeholder,
                    language=plugin.language,
                    position=CMSPlugin.objects.filter(parent=plugin).count(),
                    plugin_type=TextPlugin.__name__
                )
                child_plugin.save()
        return response


@plugin_pool.register_plugin
class CMSAllinkSEOAccordionPlugin(CMSPluginBase):
    model = AllinkSEOAccordion
    name = 'SEO Accordion Item'
    module = 'Generic'
    allow_children = True
    child_classes = settings.ALLINK_SEOACCORDION_PLUGIN_CHILD_CLASSES
    text_enabled = False

    def get_fieldsets(self, request, obj=None):
        fieldsets = (
            (None, {
                'fields': (
                    'title',
                ),
            }),
        )

        return fieldsets

    def get_render_template(self, context, instance, placeholder):
        template = 'allink_seo_accordion/item.html'
        return template



# Generated by Django 2.2.7 on 2020-01-24 10:10

from django.db import migrations, models
import djangocms_text_ckeditor.fields


class Migration(migrations.Migration):

    dependencies = [
        ('config', '0002_auto_20190716_0951'),
    ]

    operations = [
        migrations.AddField(
            model_name='config',
            name='newsletter_teaser_counter',
            field=models.IntegerField(default=4, help_text='Number which defines after how many newsentries the newsletter teaser should appear (on grid with newsletterteaser). Default is 4', verbose_name='Newsletter Teaser Counter'),
        ),
        migrations.AddField(
            model_name='configtranslation',
            name='newsletter_signup_link',
            field=models.URLField(blank=True, help_text='Link for Button on Newsletter Teaser', null=True, verbose_name='Newsletter Signup Link'),
        ),
        migrations.AlterField(
            model_name='configtranslation',
            name='newsletter_lead',
            field=djangocms_text_ckeditor.fields.HTMLField(blank=True, help_text='Teaser text in the Newsletter Teaser', null=True, verbose_name='Newsletter Signup Text'),
        ),
    ]



# Generated by Django 2.1.8 on 2019-07-02 17:20

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import djangocms_text_ckeditor.fields
import filer.fields.image
import parler.models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('cms', '0022_auto_20180620_1551'),
        migrations.swappable_dependency(settings.FILER_IMAGE_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='AllinkPageExtension',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('extended_object', models.OneToOneField(editable=False, on_delete=django.db.models.deletion.CASCADE, to='cms.Page')),
                ('og_image', filer.fields.image.FilerImageField(blank=True, help_text='og: image is used when shared on Facebook/ Twitter etc. (Min. 1200 x 630 px)<br>Page: 1. fallback is teaser_image, 2. fallback is field allink_config.default_og_image.<br>App: 1. fallback = preview_image 2. fallback is teaser_image, 3. fallback is defined in allink_config.default_og_image.<br>', null=True, on_delete=django.db.models.deletion.PROTECT, related_name='config_allinkpageextension_og_image', to=settings.FILER_IMAGE_MODEL, verbose_name='og:Image')),
                ('public_extension', models.OneToOneField(editable=False, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='draft_extension', to='config.AllinkPageExtension')),
                ('teaser_image', filer.fields.image.FilerImageField(blank=True, help_text='Optional field for teaser image. og: properties are used when shared on Facebook/ Twitter etc. (Min. 1200 x 630 px)<br>Also used as "meta" property="og:image"<br>Page: 1. fallback is allink_config.default_og_image.<br>App: 1. fallback = preview_image 2. fallback is defined in allink_config.default_og_image.<br>', null=True, on_delete=django.db.models.deletion.PROTECT, related_name='config_allinkpageextension_teaser_image', to=settings.FILER_IMAGE_MODEL, verbose_name='Teaser image')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='AllinkTitleExtension',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('og_title', models.CharField(blank=True, default='', help_text='title-tag is used when shared on Facebook/ Twitter etc.<br>Also used to overwrite "meta property="og:image.."" and title-tag<br>Page: fallback is field "title" of the page.<br>App: fallback is field "title".', max_length=255, verbose_name='og:title | <title> Tag')),
                ('og_description', models.TextField(blank=True, default='', help_text='Description is used when shared on Facebook/ Twitter etc.<br>Also used to overwrite  "meta" property="og:description" .. and "meta name="description"<br>Page: fallback is field "teaser_description" of the page, if set. Otherwise empty.<br>App: fallback is field "lead", if set. Otherwise empty.', max_length=255, verbose_name='og:description | meta description')),
                ('teaser_title', models.CharField(blank=True, default='', help_text='Page: fallback is field "title" of the page.<br>App: fallback is field "title".', max_length=255, verbose_name='Teaser title')),
                ('teaser_technical_title', models.CharField(blank=True, default='', help_text='Page: no fallback.<br>App: fallback is hardcoded per app in the teaser_dict.', max_length=255, verbose_name='Teaser technical title')),
                ('teaser_description', models.TextField(blank=True, default='', help_text='Page: no fallback.<br>App: fallback is field "lead".', verbose_name='Teaser description')),
                ('teaser_link_text', models.CharField(blank=True, default='', help_text='Page: no fallback.<br>App: fallback is hardcoded per app in the teaser_dict.', max_length=255, verbose_name='Teaser link text')),
                ('extended_object', models.OneToOneField(editable=False, on_delete=django.db.models.deletion.CASCADE, to='cms.Title')),
                ('public_extension', models.OneToOneField(editable=False, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='draft_extension', to='config.AllinkTitleExtension')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='Config',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('theme_color', models.CharField(blank=True, help_text='Theme color for Android Chrome', max_length=50, null=True, verbose_name='Theme Color')),
                ('mask_icon_color', models.CharField(blank=True, help_text='Mask icon color for safari-pinned-tab.svg', max_length=50, null=True, verbose_name='Mask icon color')),
                ('msapplication_tilecolor', models.CharField(blank=True, help_text='MS application TitleColor Field', max_length=50, null=True, verbose_name='msapplication TileColor')),
                ('google_site_verification', models.CharField(blank=True, max_length=64, null=True, verbose_name='Google Site Verification Code')),
                ('default_to_email', models.EmailField(default='itcrowd@allink.ch', max_length=255, verbose_name='Default to email')),
                ('default_from_email', models.EmailField(default='itcrowd@allink.ch', max_length=255, verbose_name='Default from email')),
                ('default_og_image', filer.fields.image.FilerImageField(blank=True, help_text='Default preview image when page/post is shared on Facebook. <br>Min. 1200 x 630 for best results.', null=True, on_delete=django.db.models.deletion.PROTECT, to=settings.FILER_IMAGE_MODEL, verbose_name='og:image')),
            ],
            options={
                'verbose_name': 'Configuration',
                'abstract': False,
            },
            bases=(parler.models.TranslatableModelMixin, models.Model),
        ),
        migrations.CreateModel(
            name='ConfigTranslation',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('language_code', models.CharField(db_index=True, max_length=15, verbose_name='Language')),
                ('default_base_title', models.CharField(blank=True, help_text='Default base title, Is also used for default base og:title when page/post is shared on Facebook. <br>If not supplied the name form Django Sites will be used instead.', max_length=50, null=True, verbose_name='Base title')),
                ('newsletter_lead', djangocms_text_ckeditor.fields.HTMLField(blank=True, help_text='Teaser text in the newsletter signup view (data usage explainer).', null=True, verbose_name='Newsletter Signup Text')),
                ('newsletter_declaration_of_consent', djangocms_text_ckeditor.fields.HTMLField(blank=True, help_text='Detailed declaration of consent.', null=True, verbose_name='Declaration of consent')),
                ('master', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='translations', to='config.Config')),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



# -*- coding: utf-8 -*-
from django.urls import reverse
from django.contrib.auth import get_permission_codename


class AllinkBaseModifierMixin:
    """
    BaseModifierMixin for basic Toolbar
    Just override model = None to specific model

    # model = <<modelclass>>
    """

    model = None

    def populate(self):
        opts = self.model._meta
        permissions_added = 0

        apps_menu = self.toolbar.get_or_create_menu(
            'apps-menu'.format(self.model._meta.model_name),
            'Apps'
        )

        menu = apps_menu.get_or_create_menu(
            '{}-menu'.format(self.model._meta.model_name),
            '{}'.format(self.model._meta.verbose_name_plural)
        )

        if self.request.user.has_perm('%s.%s' % (opts.app_label, get_permission_codename('change', opts))):
            permissions_added += 1
            url = reverse('admin:{}_{}_changelist'.format(self.model._meta.app_label, self.model._meta.model_name))
            menu.add_sideframe_item('{} List'.format(self.model._meta.verbose_name_plural), url=url)

        if self.request.user.has_perm('%s.%s' % (opts.app_label, get_permission_codename('add', opts))):
            permissions_added += 1
            url = reverse('admin:{}_{}_add'.format(self.model._meta.app_label, self.model._meta.model_name))
            menu.add_modal_item('Add new {}'.format(self.model._meta.verbose_name), url=url)

        menu, opts, permissions_added = self.custom_permissions(menu, opts, permissions_added)

        if permissions_added == 0:
            menu.disabled = True

    def custom_permissions(self, menu, opts, permissions_added):
        return menu, opts, permissions_added

    def post_template_populate(self):
        pass

    def request_hook(self):
        pass



# Generated by Django 2.2.13 on 2020-09-07 11:48

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('allink_seo_accordion', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='allinkseoaccordioncontainerplugin',
            name='is_seo_faq',
            field=models.BooleanField(default=False, help_text='Enable to display accordion contents as questions/answers in search engine result pages', verbose_name='Enable SEO FAQ schema'),
        ),
    ]



# Generated by Django 2.2.5 on 2019-10-16 08:04

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('allink_legacy_redirect', '0002_auto_20190723_1107'),
    ]

    operations = [
        migrations.AlterField(
            model_name='allinklegacylink',
            name='language',
            field=models.CharField(max_length=200, null=True, verbose_name='Language'),
        ),
    ]



# -*- coding: utf-8 -*-
from urllib.parse import urlparse
import phonenumbers
from importlib import import_module

from django.conf import settings
from django.db import models
from django.urls import NoReverseMatch
from django.urls import reverse
from django.utils.translation import ugettext_lazy as _
from django.core.exceptions import ValidationError
from django.contrib.postgres.fields import ArrayField
from django.utils.encoding import force_text
from django.utils.functional import cached_property
from filer.fields.image import FilerImageField
from filer.fields.file import FilerFileField
from model_utils.models import TimeStampedModel
from phonenumber_field.modelfields import PhoneNumberField
from djangocms_attributes_field.fields import AttributesField
from cms.models.fields import PageField
from allink_core.core.loading import get_model
from allink_core.core.models.choices import (
    SALUTATION_CHOICES, TARGET_CHOICES, NEW_WINDOW, SOFTPAGE, FORM_MODAL, IMAGE_MODAL, DEFAULT_MODAL, BLANK_CHOICE, )
from allink_core.core_apps.allink_categories.models import AllinkCategory
from allink_core.core.models.managers import AllinkCategoryModelManager
from allink_core.core.models.fields import ZipCodeField
from allink_core.core.utils import get_additional_choices


__all__ = [
    'AllinkStatusFieldsModel',
    'AllinkTimeFramedModel',
    'AllinkCategoryFieldsModel',
    'AllinkSEOFieldsModel',
    'AllinkSEOTranslatedFieldsModel',
    'AllinkTeaserFieldsModel',
    'AllinkTeaserTranslatedFieldsModel',
    'AllinkAddressFieldsModel',
    'AllinkContactFieldsModel',
    'AllinkInternalLinkFieldsModel',
    'AllinkLinkFieldsModel',
    'AllinkSimpleRegistrationFieldsModel',
]


class AllinkStatusFieldsModel(models.Model):
    """
    Base class for status field

    hint: Just override STATUS_CHOICES or STATUS_DEFAULT in you model, if needed.
    """
    ACTIVE = 1
    INACTIVE = 2
    STATUS_DEFAULT = ACTIVE

    STATUS_CHOICES = [
        (ACTIVE, 'active'),
        (INACTIVE, 'inactive')
    ]
    status = models.IntegerField('status', choices=STATUS_CHOICES, default=STATUS_DEFAULT)

    class Meta:
        abstract = True


class AllinkTimeFramedModel(models.Model):
    """
    Base class for timeframed models
    We don't use django_modelutils, because they add a timeframed default manager.
    And this is not compatible with parler's TranslatableQuerySet.

    """
    start = models.DateTimeField('start', null=True, blank=True)
    end = models.DateTimeField('end', null=True, blank=True)

    class Meta:
        abstract = True


class AllinkCategoryFieldsModel(models.Model):
    """
    Base class for apps with categories
    """
    categories = models.ManyToManyField(
        AllinkCategory,
        blank=True
    )

    objects = AllinkCategoryModelManager()

    class Meta:
        abstract = True

    @cached_property
    def fetch_categories(self):
        return self.categories.all()

    @classmethod
    def get_relevant_categories(cls):
        """
        returns a queryset of all relevant categories for a the model_name
        """
        result = AllinkCategory.objects.none()
        for root in AllinkCategory.get_root_nodes().filter(model_names__contains=[cls._meta.model_name]):
            result |= root.get_descendants()
        return result

    @classmethod
    def get_can_have_categories(cls):
        return cls._meta.model_name in dict(settings.PROJECT_APP_MODEL_WITH_CATEGORY_CHOICES)


class AllinkSEOFieldsModel(models.Model):
    """
    Base class for all non translated SEO fields
    """
    og_image = FilerImageField(
        verbose_name='og:Image',
        on_delete=models.PROTECT,
        help_text=
            'og: image is used when shared on Facebook/ Twitter etc. (Min. 1200 x 630 px)<br>'
            'Page: 1. fallback is teaser_image, 2. fallback is field allink_config.default_og_image.<br>'
            'App: 1. fallback = preview_image 2. fallback is teaser_image, 3. '
            'fallback is defined in allink_config.default_og_image.<br>',
        related_name='%(app_label)s_%(class)s_og_image',
        blank=True,
        null=True
    )

    class Meta:
        abstract = True


class AllinkSEOTranslatedFieldsModel(models.Model):
    """
    Base class for all translated SEO fields

    This is supposed to be used in combination with TranslatedFieldsModel
    e.g.: ...(AllinkSEOTranslatedFieldsModel, TranslatedFieldsModel)
    """
    og_title = models.CharField(
        verbose_name='og:title | <title> Tag',
        max_length=255,
        help_text='title-tag is used when shared on Facebook/ Twitter etc.<br>'
                  'Also used to overwrite "meta property="og:image.."" and title-tag<br>'
                  'Page: fallback is field "title" of the page.<br>'
                  'App: fallback is field "title".',
        blank=True,
        default=''
    )
    og_description = models.TextField(
        verbose_name='og:description | meta description',
        max_length=255,
        help_text='Description is used when shared on Facebook/ Twitter etc.<br>'
                  'Also used to overwrite  "meta" property="og:description" .. and "meta name="description"<br>'
                  'Page: fallback is field "teaser_description" of the page, if set. Otherwise empty.<br>'
                  'App: fallback is field "lead", if set. Otherwise empty.',
        blank=True,
        default=''
    )

    class Meta:
        abstract = True


class AllinkTeaserFieldsModel(models.Model):
    """
    Base class for all teaser fields

    use this together with: AllinkTeaserMixin
    """
    teaser_image = FilerImageField(
        verbose_name='Teaser image',
        on_delete=models.PROTECT,
        help_text=
            'Optional field for teaser image. og: properties are used when shared on Facebook/ Twitter etc. '
            '(Min. 1200 x 630 px)<br>'
            'Also used as "meta" property="og:image"<br>'
            'Page: 1. fallback is allink_config.default_og_image.<br>'
            'App: 1. fallback = preview_image 2. fallback is defined in allink_config.default_og_image.<br>',
        related_name='%(app_label)s_%(class)s_teaser_image',
        blank=True,
        null=True
    )

    class Meta:
        abstract = True


class AllinkTeaserTranslatedFieldsModel(models.Model):
    """
    This is supposed to be used in combination with TranslatedFieldsModel
    e.g.: ...(AllinkTeaserTranslatedFieldsModel, TranslatedFieldsModel)

    use this together with: AllinkTeaserMixin
    """
    teaser_title = models.CharField(
        'Teaser title',
        help_text='Page: fallback is field "title" of the page.<br>'
                  'App: fallback is field "title".',
        max_length=255,
        blank=True,
        default=''
    )
    teaser_technical_title = models.CharField(
        'Teaser technical title',
        help_text='Page: no fallback.<br>'
                  'App: fallback is hardcoded per app in the teaser_dict.',
        max_length=255,
        blank=True,
        default=''
    )
    teaser_description = models.TextField(
        'Teaser description',
        help_text='Page: no fallback.<br>'
                  'App: fallback is field "lead".'
                  'Please only use 80 to 120 characters. Best results will be achieved with around 100 characters.',
        blank=True,
        default=''
    )
    teaser_link_text = models.CharField(
        'Teaser link text',
        help_text='Page: no fallback.<br>'
                  'App: fallback is hardcoded per app in the teaser_dict.',
        max_length=255,
        blank=True,
        default=''
    )

    class Meta:
        abstract = True


class AllinkAddressFieldsModel(models.Model):
    """
    Base class for all address related fields

    TODO: country might need to be moved to a TranslatedFieldsModel
    or we should switch to a django-countries. CountryField
    """
    street = models.CharField(
        _('Street'),
        max_length=255,
        blank=True,
        null=True
    )
    street_nr = models.CharField(
        _('Street Nr.'),
        max_length=50,
        blank=True,
        null=True
    )
    street_additional = models.CharField(
        _('Address Additional'),
        max_length=255,
        blank=True,
        null=True
    )
    zip_code = ZipCodeField(
        _('Zip Code'),
        blank=True,
        null=True
    )
    place = models.CharField(
        _('Place'),
        max_length=255,
        blank=True,
        null=True
    )
    country = models.CharField(
        _('Country'),
        max_length=255,
        blank=True,
        null=True
    )

    class Meta:
        abstract = True


class AllinkContactFieldsModel(models.Model):
    """
    Base class for all contact fields
    """
    phone = PhoneNumberField(
        _('Phone'),
        help_text=_(
            'We automatically handle phone number formatting, '
            'Please provide the number in the following format "+41 43 123 45 67".'),
        blank=True,
        null=True
    )
    mobile = PhoneNumberField(
        _('Mobile'),
        help_text=_(
            'We automatically handle phone number formatting, '
            'Please provide the number in the following format "+41 43 123 45 67".'),
        blank=True,
        null=True
    )
    fax = PhoneNumberField(
        _('Fax'),
        help_text=_(
            'We automatically handle phone number formatting, '
            'Please provide the number in the following format "+41 43 123 45 67".'),
        blank=True,
        null=True
    )
    email = models.EmailField(
        _('Email'),
        blank=True,
        default=''
    )
    website = models.URLField(
        _('Website'),
        blank=True,
        null=True
    )

    class Meta:
        abstract = True

    @cached_property
    def phone_formatted(self):
        if self.phone:
            x = phonenumbers.parse(str(self.phone), None)
            return str(phonenumbers.format_number(x, phonenumbers.PhoneNumberFormat.INTERNATIONAL))

    @cached_property
    def mobile_formatted(self):
        if self.mobile:
            x = phonenumbers.parse(str(self.mobile), None)
            return str(phonenumbers.format_number(x, phonenumbers.PhoneNumberFormat.INTERNATIONAL))

    @cached_property
    def fax_formatted(self):
        if self.fax:
            x = phonenumbers.parse(str(self.fax), None)
            return str(phonenumbers.format_number(x, phonenumbers.PhoneNumberFormat.INTERNATIONAL))

    @cached_property
    def website_clean(self):
        if self.website:
            website = urlparse(self.website)
            domain = '{uri.netloc}'.format(uri=website)
            return domain.replace('www.', '')


class AllinkInternalLinkFieldsModel(models.Model):
    """
    Base class for all internal link related fields
    """
    #  Page redirect
    link_page = PageField(
        verbose_name='New Page',
        null=True,
        on_delete=models.PROTECT,
        help_text='If provided, overrides the external link and New Apphook-Page.',
    )
    #  Fields for app redirect
    link_apphook_page = PageField(
        verbose_name='New Apphook-Page',
        null=True,
        on_delete=models.PROTECT,
        help_text='If provided, overrides the external link.',
        related_name='%(app_label)s_%(class)s_app_legacy_redirects'
    )
    link_object_id = models.IntegerField(
        null=True,
        help_text='To which object directs the url.'
    )
    link_model = models.CharField(
        null=True,
        max_length=300,
        help_text='Dotted Path to referenced Model'
    )
    link_url_name = models.CharField(
        null=True,
        max_length=64,
        help_text='Name of the App-URL to use.'
    )
    link_url_kwargs = ArrayField(
        models.CharField(
            max_length=50,
            blank=True,
            null=True
        ),
        blank=True,
        null=True,
        help_text='Keyword arguments used to reverse url.'
    )

    class Meta:
        abstract = True

    @cached_property
    def link(self):
        if self.link_page:
            link = self.link_page.get_absolute_url()
        elif self.link_apphook_page:
            try:
                obj_model = get_model(self.link_model.split('.')[-3], self.link_model.split('.')[-1])
                obj = obj_model.objects.get(id=self.link_object_id)
                url_kwargs = {key: getattr(obj, key) for key in self.link_url_kwargs}
                url_name = '{}:{}'.format(self.link_apphook_page.application_namespace, self.link_url_name)
                link = reverse(url_name, kwargs=url_kwargs)
            except KeyError:
                link = ''
        else:
            link = ''
        return link

    @cached_property
    def link_object(self):
        if self.link_page:
            link_obj = self.link_page
        elif self.link_apphook_page:
            try:
                obj_module = import_module('.'.join(self.link_model.split('.')[:-1]))
                obj_model = getattr(obj_module, self.link_model.split('.')[-1])
                link_obj = obj_model.objects.get(id=self.link_object_id)
            except KeyError:
                link_obj = None
        else:
            link_obj = None
        return link_obj

    @cached_property
    def is_page_link(self):
        if self.link_page:
            return True
        else:
            return False


class AllinkLinkFieldsModel(AllinkInternalLinkFieldsModel):
    """
    Base class for all the link related fields (including internal link fields)
    """
    link_url = models.URLField(
        verbose_name='External link',
        blank=True,
        default='',
        help_text='Provide a valid URL to an external website.',
        max_length=500,
    )
    link_mailto = models.EmailField(
        verbose_name='Email address',
        blank=True,
        null=True,
        max_length=255,
    )
    link_phone = models.CharField(
        verbose_name='Phone',
        blank=True,
        null=True,
        max_length=255,
    )
    link_anchor = models.CharField(
        verbose_name='Anchor',
        max_length=255,
        blank=True,
        help_text='Appends the value only after the internal or external link.<br>'
                    'Do <strong>not</strong> include a preceding "#" symbol.',
    )
    link_target = models.IntegerField(
        'Link Target',
        choices=TARGET_CHOICES,
        null=True,
        blank=True
    )
    link_file = FilerFileField(
        verbose_name='file',
        on_delete=models.PROTECT,
        null=True,
        blank=True
    )
    link_special = models.CharField(
        verbose_name='Special Links',
        max_length=255,
        blank=True,
        null=True
    )
    link_attributes = AttributesField(
        verbose_name='Attributes',
        blank=True,
        excluded_keys=['class', 'href', 'target'],
    )

    class Meta:
        abstract = True

    @cached_property
    def new_window_enabled(self):
        return True if self.link_target == NEW_WINDOW and not self.form_modal_enabled \
                       and not self.softpage_enabled else False

    @cached_property
    def softpage_enabled(self):
        return True if self.link_target == SOFTPAGE else False

    @cached_property
    def form_modal_enabled(self):
        return True if self.link_target == FORM_MODAL else False

    @cached_property
    def image_modal_enabled(self):
        return True if self.link_target == IMAGE_MODAL else False

    @property
    def default_modal_enabled(self):
        return True if self.link_target == DEFAULT_MODAL else False

    @classmethod
    def get_link_special_choices(self):
        return BLANK_CHOICE + get_additional_choices('BUTTON_LINK_SPECIAL_LINKS_CHOICES')

    @cached_property
    def link_url_typed(self):
        internal_link = self.link
        if internal_link:
            link = internal_link
        elif self.link_url:
            link = self.link_url
        elif self.link_phone:
            link = 'tel:{}'.format(self.link_phone.replace(' ', ''))
        elif self.link_mailto:
            link = 'mailto:{}'.format(self.link_mailto)
        elif self.link_file:
            link = self.link_file.url
        elif self.link_special:
            try:
                """
                because we are always in a plugin (e.g Button, Image..)
                and plugins can appear more than once on the same page,
                the urls should always pass a plugin_id (not all urls do at the moment)
                """
                link = reverse(self.link_special, kwargs={'plugin_id': self.id})
            except NoReverseMatch:
                link = reverse(self.link_special)
        else:
            link = ''
        if self.link_anchor:
            link += '#{}'.format(self.link_anchor)
        return link

    def clean(self):
        super(AllinkLinkFieldsModel, self).clean()
        field_names = (
            'link_url',
            'link_page',
            'link_apphook_page',
            'link_mailto',
            'link_phone',
            'link_file',
        )
        anchor_field_name = 'link_anchor'
        field_names_allowed_with_anchor = (
            'link_url',
            'link_page',
            'link_apphook_page',
            'link_file',
        )

        anchor_field_verbose_name = force_text(self._meta.get_field(anchor_field_name).verbose_name)
        anchor_field_value = getattr(self, anchor_field_name)

        link_fields = {
            key: getattr(self, key)
            for key in field_names
        }
        link_field_verbose_names = {
            key: force_text(self._meta.get_field(key).verbose_name)
            for key in link_fields.keys()
        }
        provided_link_fields = {
            key: value
            for key, value in link_fields.items()
            if value
        }

        if anchor_field_value:
            for field_name in provided_link_fields.keys():
                if field_name not in field_names_allowed_with_anchor:
                    error_msg = '%(anchor_field_verbose_name)s is not allowed together with %(field_name)s' % {
                        'anchor_field_verbose_name': anchor_field_verbose_name,
                        'field_name': link_field_verbose_names.get(field_name)
                    }
                    raise ValidationError({
                        anchor_field_name: error_msg,
                        field_name: error_msg,
                    })


class AllinkSimpleRegistrationFieldsModel(TimeStampedModel):
    """
    Base class for all registration related fields
    """
    salutation = models.IntegerField(
        _('Salutation'),
        choices=SALUTATION_CHOICES,
        null=True
    )

    first_name = models.CharField(
        _('First Name'),
        max_length=255,
        null=True
    )

    last_name = models.CharField(
        _('Last Name'),
        max_length=255,
        null=True
    )

    email = models.EmailField(
        _('Email'),
        null=True
    )

    company_name = models.CharField(
        _('Company'),
        max_length=255,
        blank=True,
        null=True
    )
    phone = models.CharField(
        _('Phone'),
        max_length=30,
        blank=True,
        null=True
    )

    message = models.TextField(
        _('Message'),
        max_length=255,
        blank=True,
        null=True
    )

    class Meta:
        abstract = True
